﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MCSPlugins;
using Microsoft.Xrm.Sdk;
using System.ServiceModel;
using mcsScheduling;
using Microsoft.Xrm.Sdk.Query;
namespace CVTPlugins
{
    class CvtComponentCreateUpdatePostStageRunner : PluginRunner
    {
        public CvtComponentCreateUpdatePostStageRunner (IServiceProvider serviceProvider) : base(serviceProvider){}
        /// <summary>
        /// This method simply copies the Guid from the EntityReference (aka lookup) Related Resource to the Parent Resource Identifier field
        /// </summary>
        public void Execute()
        {
            try
            {
                TracingService.Trace("at the top");

                Logger.setMethod = "Execute";
                Logger.WriteDebugMessage("Beginning Component Guid Expose");

                if (!PluginExecutionContext.InputParameters.Contains("Target"))
                    return;

                var component = (Entity)PluginExecutionContext.InputParameters["Target"];
                if (component.LogicalName != cvt_component.EntityLogicalName)
                    throw new InvalidPluginExecutionException("Entity is not of Type Component");

                CheckTSSResource(component);

                //Since this is registered Pre-Stage, passing the value into the actual object which is being created/updated will result in the update being passed into the create/update event (no service call is needed)
                component.Attributes["cvt_parentresourceidentifier"] = ((EntityReference)(component.Attributes["cvt_relatedresourceid"])).Id.ToString();

                Logger.WriteDebugMessage("Ending");
                //End the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Ending : {0}", GetType()));
            }
            catch(FaultException<OrganizationServiceFault> ex){
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage);
            }
            catch(Exception ex){
                if (ex.Message.StartsWith("custom")){
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
            }
        }
        internal void CheckTSSResource(Entity component)
        {
            using (var srv = new Xrm(OrganizationService))
            {
                Logger.WriteDebugMessage("Check for parent TSS Resource");
                if (component.Contains("cvt_relatedresourceid"))
                {
                    Guid parentResourceId = ((EntityReference)(component.Attributes["cvt_relatedresourceid"])).Id;
                    mcs_resource parentResource = (mcs_resource)OrganizationService.Retrieve(mcs_resource.EntityLogicalName, parentResourceId, new ColumnSet(true));
                    if (parentResource.mcs_Type != null && parentResource.mcs_Type.Value != 251920002)
                    {
                        Logger.WriteDebugMessage("Parent TSS Resource is not Technology");
                        throw new InvalidPluginExecutionException("customTSS Resource is not a Technology.");
                    }
                    else
                        Logger.WriteDebugMessage("Parent TSS Resource is Technology, so allow create");
                }
                else //Field Requirement should prevent logic from entering this branch
                    throw new InvalidPluginExecutionException("customTSS Resource is missing.");
            }
        }

        #region Additional Interface Methods/Properties
        /// <summary>
        /// Gets the Primary Entity which is triggering this plugin - not needed for this plugin except to satisfy the Abstract Class
        /// </summary>
        /// <returns>Entity Object which triggered plugin</returns>
        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        /// <summary>
        /// Gets the Entity pre-image - not needed for this plugin except to satisfy the Abstract Class
        /// </summary>
        /// <returns></returns>
        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PreEntityImages["pre"];
        }

        /// <summary>
        /// Used for Debugging - turns on or off creation of log records for this particular entity
        /// </summary>
        public override string McsSettingsDebugField
        {
            get { return "cvt_componentplugin"; }
        }
        #endregion
    }
}
