﻿using System.Linq;
using mcsScheduling;
using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using System.Collections.Generic;

namespace CVTPlugins
{
    public class CvtGroupPatientDeletePreStageRunner : MCSPlugins.PluginRunner
    {
        #region Constructor
        public CvtGroupPatientDeletePreStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods/Properties
        internal void Execute()
        {
            try
            {             
                TracingService.Trace("at the top");

                Logger.setMethod = "Execute";
                //start the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Starting : {0}", GetType()));
                Logger.WriteDebugMessage("Starting");

                Logger.WriteDebugMessage(String.Format("At the top of: {0}", GetType()));
               
                //checks to make sure the registration is correct
                if (PluginExecutionContext.InputParameters.Contains("Target") &&
                    PluginExecutionContext.InputParameters["Target"] is EntityReference)
                {
                        Logger.WriteGranularTimingMessage("Starting UpdateCapacity");
                        //Create the Patient Site Rooms associated with this Service Activity
                        UpdateCapacity(PluginExecutionContext.PrimaryEntityId);
                        Logger.WriteGranularTimingMessage("Ending UpdateCapacity");
                  //  }
                }
                Logger.WriteDebugMessage("Ending");
                //End the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Ending : {0}", GetType()));
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }
        
        internal void UpdateCapacity(Guid thisPatient)
        {
            try
            {
                using (var srv = new Xrm(OrganizationService))
                {                              
                    //Looking up the Patient being deleted. 
                    var DeletedPatient = srv.CreateQuery<cvt_grouppatient>().FirstOrDefault(gp =>
                            gp.Id == thisPatient);
                    
                    Logger.WriteDebugMessage("Patient being deleted: " + DeletedPatient.cvt_name);

                    // Look up the Pat Site Room to be updated
                    Logger.WriteDebugMessage("Loading the PatSiteRoom");

                    var PatSiteRoom = srv.CreateQuery<mcs_patientsiteroom>().FirstOrDefault(c =>
                            c.Id == DeletedPatient.cvt_relatedpatientsiteroomid.Id);
                    if (PatSiteRoom == null)
                        return;
                   
                    //Look up that Patients related to the Patient Site Room (minus the one we are deleting).
                    Logger.WriteDebugMessage("Creating collection of Patients.");
                    var PatSitePatients = from psp in srv.cvt_grouppatientSet
                                       where (psp.cvt_relatedpatientsiteroomid.Id == DeletedPatient.cvt_relatedpatientsiteroomid.Id)
                                           && (psp.Id != thisPatient)
                                       select psp;

                    foreach (var psp in PatSitePatients)
                    {
                        Logger.WriteDebugMessage("Pat Site Patient: " + psp.cvt_name);
                    }  

                    //If there are no related Patients, set Patient count to 0, otherwise loop through and count the related Patients. 
                    var patientCount = 0;
                    if (PatSitePatients == null)
                    {
                        patientCount += 0;
                    }
                    else
                    {
                        foreach (var psp in PatSitePatients)
                        {
                            patientCount += 1;
                        }
                        //Will Subtract one for Patient that is being deleted.
                        //patientCount = patientCount - 1;
                    }

                    Logger.WriteDebugMessage("pat count: " + patientCount);


                    var Capacity = PatSiteRoom.cvt_capacity;
                    var remainingCapacity = Capacity - patientCount;

                    var updatePatSiteRoom = new mcs_patientsiteroom()
                    {
                        Id = DeletedPatient.cvt_relatedpatientsiteroomid.Id,
                        cvt_patients = patientCount,
                        cvt_remainingcapacity = remainingCapacity
                    };
                    OrganizationService.Update(updatePatSiteRoom);
                   // srv.SaveChanges();
                    
                    //Updating the Service Activity with the total # of Patients
                    
                    var ServiceActivityPatients = from sap in srv.cvt_grouppatientSet
                                                  where (sap.cvt_relatedserviceactivityid.Id == DeletedPatient.cvt_relatedserviceactivityid.Id)
                                                  && (sap.Id != thisPatient)
                                                  select sap;
                    foreach (var sap in ServiceActivityPatients)
                    {
                        Logger.WriteDebugMessage("Service Activity Patient: " + sap.cvt_name);
                    }  
                    var SApatientCount = 0;
                    if (ServiceActivityPatients == null)
                    {
                        SApatientCount += 0;
                    }
                    else
                    {
                        foreach (var p in ServiceActivityPatients)
                        {
                            SApatientCount += 1;
                        }
                        //Will Subtract one for Patient that is being deleted. 
                        //SApatientCount = SApatientCount - 1;
                    }
                    Logger.WriteDebugMessage("SA pat count: " + SApatientCount);

                    //Update the Patient Count and Remaining Capcity on the Related Service Activity. 
                    var RelatedServiceActivity = srv.CreateQuery<ServiceAppointment>().FirstOrDefault(rsa =>
                            rsa.Id == DeletedPatient.cvt_relatedserviceactivityid.Id);

                    var currentSACapacity = RelatedServiceActivity.mcs_capacity;
                    var remainingSACapacity = currentSACapacity - SApatientCount;

                    var updateServiceActivity = new ServiceAppointment()
                    {
                        Id = DeletedPatient.cvt_relatedserviceactivityid.Id,
                        cvt_patientcount = SApatientCount,
                        mcs_remainingcapacity = remainingSACapacity
                    };
                    
                    //Finally: Update the 'customers' field on the related Service Activity so we can accurately report on which Service Activity each Patient is realated to. 
                    //Creating a new Activity Party List and pulling all Group Patient / Patients and adding to the customer field on the Service Activity.                                       
                    List<ActivityParty> Patients = new List<ActivityParty>();
                    if (SApatientCount > 0)
                    {
                        foreach (var p in ServiceActivityPatients)
                        {
                            ActivityParty party = new ActivityParty();
                            EntityReference PatientReference = p.cvt_relatedpatientid;
                            party.PartyId = PatientReference;
                            Patients.Add(party);
                        }
                        if (Patients.Count > 0)
                        {
                            ActivityParty[] newPatientList = new ActivityParty[Patients.Count];
                            for (int i = 0; i < Patients.Count; i++)
                            {
                                newPatientList[i] = (ActivityParty)Patients[i];
                            }
                            updateServiceActivity.Customers = newPatientList;
                        }
                    }
                     
                    //Update the Service Activity with new Capacity values and add patients to the customers field.
                    OrganizationService.Update(updateServiceActivity);
                    srv.SaveChanges();
                    Logger.WriteDebugMessage("Updated the Service Activity Patient Count.");                  
                }
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
            }
            catch (Exception ex)
            {
                Logger.WriteToFile(ex.Message);
            }
        }
        #endregion

        #region Additional Interface Methods/Properties
        public override string McsSettingsDebugField
        {
            get { return "cvt_grouppatientplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.PreEntityImages["pre"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PreEntityImages["pre"];
        }
        #endregion
    }
}