﻿using System.Linq;
using mcsScheduling;

using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;

namespace CVTPlugins
{
    public class cvtMasterTSACreatePostStageRunner : MCSPlugins.PluginRunner
    {
        #region Constructor
        public cvtMasterTSACreatePostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods
        internal void Execute()
        {
            try
            {
                TracingService.Trace("at the top");
                Logger.setMethod = "Execute";
                Logger.WriteTxnTimingMessage("Starting");
                Logger.WriteDebugMessage(String.Format("At the top of: {0}", GetType()));
                if (!PluginExecutionContext.InputParameters.Contains("Target") || !(PluginExecutionContext.InputParameters["Target"] is Entity))
                    return;

                var entity = (Entity)PluginExecutionContext.InputParameters["Target"];

                if (entity.LogicalName != cvt_mastertsa.EntityLogicalName)
                    throw new Exception("Target entity is not of type cvt_mastertsa");

                var mtsa = entity.ToEntity<cvt_mastertsa>();
                Logger.WriteGranularTimingMessage("Starting UpdateMTSA");
                UpdateMTSA(PluginExecutionContext.PrimaryEntityId);

                Logger.setMethod = "Execute";
                Logger.WriteGranularTimingMessage("Ending UpdateMTSA");                
                Logger.WriteTxnTimingMessage("Ending");
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
            }
            catch (Exception ex)
            {
                Logger.WriteToFile(ex.Message);
            }
        }

        internal void UpdateMTSA(Guid mtsaId)
        {
            Logger.setMethod = "UpdateMTSA";
            Logger.WriteDebugMessage("starting UpdateMTSA");
            using (var srv = new Xrm(OrganizationService))
            {
                try
                {
                    if (mtsaId == null) { return; }

                    var mtsa = srv.cvt_mastertsaSet.FirstOrDefault(i => i.Id == mtsaId);
                    var getProvResources = from provGroups in srv.cvt_providerresourcegroupSet
                                           where provGroups.cvt_RelatedMasterTSAId.Id == mtsaId
                                           where provGroups.statecode == 0
                                           select new
                                           {
                                               provGroups.Id,
                                               provGroups.cvt_RelatedResourceId,
                                               provGroups.cvt_resourcespecguid,
                                               provGroups.cvt_TSAResourceType,
                                               provGroups.cvt_Type,
                                               provGroups.cvt_RelatedUserId,
                                               provGroups.cvt_RelatedResourceGroupid
                                           };

                    //grab all the provider groups
                    string provSiteVCs = null;
                    string providers = null;
                    foreach (var provGroups in getProvResources)
                    {
                        if ((provGroups.cvt_Type != null) && (provGroups.cvt_Type.Value == 251920000))
                            provSiteVCs += provGroups.cvt_RelatedResourceId.Name.ToString() + " ; ";
                        if ((provGroups.cvt_TSAResourceType.Value == 2 || ((provGroups.cvt_TSAResourceType.Value == 0) && (provGroups.cvt_Type.Value == 99999999))))
                        {
                            if (provGroups.cvt_RelatedUserId != null)
                                providers += provGroups.cvt_RelatedUserId.Name.ToString() + " ; ";
                            if (provGroups.cvt_RelatedResourceGroupid != null)
                                providers += provGroups.cvt_RelatedResourceGroupid.Name.ToString() + " ; ";
                        }
                    }

                    var updateMTSA = new Entity("cvt_mastertsa") { Id = mtsaId };
                    updateMTSA["cvt_providers"] = providers;
                    updateMTSA["cvt_providersitevistaclinics"] = provSiteVCs;
                    OrganizationService.Update(updateMTSA);
                    Logger.WriteDebugMessage("MTSA Updated");
                }

                catch (FaultException<OrganizationServiceFault> ex)
                {
                    Logger.WriteToFile(ex.Message);
                }
                catch (Exception ex)
                {
                    Logger.WriteToFile(ex.Message);
                }
            }
        }
        #endregion
        
        #region Additional Interface Methods
        public override string McsSettingsDebugField
        {
            get { return "mcs_mastertsaplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }
        #endregion
    }
}