﻿using System;
using System.ServiceModel;
using Microsoft.Xrm.Sdk;
using mcsScheduling;
using System.Linq;

namespace CVTPlugins
{
    public class McsServicesDeletePreStageRunner : MCSPlugins.PluginRunner
    {
        #region Constructor
        public McsServicesDeletePreStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods/Properties
        internal void Execute()
        {
            try
            {
                Logger.setMethod = "Execute";

                Logger.WriteTxnTimingMessage("Starting");

                if (PluginExecutionContext.InputParameters.Contains("Target") &&
                    PluginExecutionContext.InputParameters["Target"] is EntityReference)
                {
                                      
                    Logger.WriteDebugMessage(String.Format("At the top of:{0}", GetType()));
                                                     
                    Logger.WriteGranularTimingMessage("Starting TSA");                   
                    DeleteTSA(PluginExecutionContext.PrimaryEntityId);

                    Logger.WriteGranularTimingMessage("Ending TSA");
                }

                Logger.setMethod = "Execute";

                Logger.WriteTxnTimingMessage("Ending");

            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage + ":" + ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));

                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage + ":" + ex.Message);
                }
            }
        }

        internal void DeleteTSA(Guid thisId)
        {
            Logger.WriteDebugMessage("Starting Delete TSA");
           
            using (var srv = new Xrm(OrganizationService))
            {
                //Starting Validation Checks before deleting the TSA. 
                Logger.setMethod = "Delete TSA";
                
                var thisTSA = srv.mcs_servicesSet.FirstOrDefault(i => i.Id == thisId);

                if (thisTSA.mcs_RelatedServiceId == null) { return; }

                var systemService = srv.ServiceSet.FirstOrDefault(i => i.Id == thisTSA.mcs_RelatedServiceId.Id);
                //Checking to see if any Patient Site Resources have this target MCS Resource associated with it.
                var ServiceActivity = srv.ServiceAppointmentSet.FirstOrDefault(i => i.mcs_relatedtsa.Id == thisId);                                  
                //If an associated Patient Site Resource does exist, we will throw an exception with a message and stop the delete of the MCS Resource, to prevent orphan data. 
                if (ServiceActivity != null)
                {                  
                   // Will Prevent deletion of TSA's if any Service Activities exists for now. Pending input. 
                    // int SAState = McsHelper.getOptionSetValue("statecode");
                   // if ((SAState != 1) || (SAState != 2))
                   // {
                        Logger.WriteDebugMessage("Service Activities:" + ServiceActivity.Subject);
                        throw new InvalidPluginExecutionException("customPlease check Left Nav on TSA form for Service Activities this TSA is associated with. TSA cannot be deleted while associated open or scheduled Service Activities exist.");
                   // }
                }

                //If there are no related Service Activites, we will delete the realted System Service. 
                OrganizationService.Delete(systemService.LogicalName, systemService.Id);
                Logger.WriteDebugMessage("System Service Deleted");

            }
        }

        public override string McsSettingsDebugField
        {
            get { return "mcs_tsaplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.PreEntityImages["pre"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PreEntityImages["pre"];
       }
    
        #endregion
    }
}
