﻿using System.Linq;
using mcsScheduling;
using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;

namespace CVTPlugins
{
    /// <summary>
    /// Purpose:  This is not current registered pending business decision
    /// </summary>
    public class CvtPatGroupUpdatePostStageRunner : MCSPlugins.PluginRunner
    {
        #region Constructor
        public CvtPatGroupUpdatePostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods/Properties
        internal void Execute()
        {
            try
            {
                TracingService.Trace("at the top");
                Logger.setMethod = "Execute";
                //start the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Starting : {0}", GetType()));
                Logger.WriteDebugMessage(String.Format("Starting : {0}", GetType()));

                //checks to make sure the registration is correct
                if (!PluginExecutionContext.InputParameters.Contains("Target") || !(PluginExecutionContext.InputParameters["Target"] is Entity))
                    return;
                var entity = (Entity)PluginExecutionContext.InputParameters["Target"];

                if (entity.LogicalName != cvt_patientresourcegroup.EntityLogicalName)
                    throw new Exception("Target entity is not of type cvt_patientresourcegroup");

                MaintainResourceGroup(PluginExecutionContext.PrimaryEntityId);

                Logger.WriteDebugMessage("Ending");
                //End the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Ending : {0}", GetType()));
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }

        internal void MaintainResourceGroup(Guid thisId)
        {
            Logger.WriteDebugMessage("starting MaintainResourceGroup");
            var whereami = "top";
            using (var srv = new Xrm(OrganizationService))
            {
                try
                {
                    Logger.setMethod = "MaintainResourceGroup";
                    var builder = new System.Text.StringBuilder("<Constraints>");
                    builder.Append("<Constraint>");
                    builder.Append("<Expression>");
                    var getResources = from patGroups in srv.cvt_patientresourcegroupSet
                                       where patGroups.cvt_patientresourcegroupId.Value == thisId && patGroups.statecode == 0
                                       select new
                                       {
                                           patGroups.cvt_TSAResourceType,
                                           patGroups.cvt_RelatedResourceGroupid,
                                           patGroups.cvt_RelatedResourceId,
                                           patGroups.cvt_RelatedUserId                                           
                                       };
                    var count = 0;
                    foreach (var patGroups in getResources)
                    {
                        builder.Append(count > 0 ? " || resource[\"Id\"] == " : "<Body>resource[\"Id\"] == ");
                        switch (patGroups.cvt_TSAResourceType.Value)
                        {
                            case 0: //resource group
                                Logger.WriteDebugMessage("About to get resourceGroup");
                                var resourceGroup = srv.mcs_resourcegroupSet.FirstOrDefault(i => i.Id == patGroups.cvt_RelatedResourceGroupid.Id);
                                if (resourceGroup == null) return;
                                Logger.WriteDebugMessage("got resourceGroup");
                                builder.Append(resourceGroup.mcs_RelatedResourceGroupId.Id.ToString("B"));
                                break;
                            case 1: //single resource
                                Logger.WriteDebugMessage("About to get resource");
                                var resource = srv.mcs_resourceSet.FirstOrDefault(i => i.Id == patGroups.cvt_RelatedResourceId.Id);
                                if (resource == null) return;
                                Logger.WriteDebugMessage("got resource");
                                builder.Append(resource.mcs_relatedResourceId.Id.ToString("B"));
                                break;
                            case 2: //single provider
                                builder.Append(patGroups.cvt_RelatedUserId.Id.ToString("B"));
                                break;
                            case 3: //single person
                                builder.Append(patGroups.cvt_RelatedUserId.Id.ToString("B"));
                                break;
                        }                                                
                        count += 1;
                    }
                    if (count == 0)
                    {
                        builder.Append("<Body>false");
                    }

                    builder.Append("</Body>");
                    builder.Append("<Parameters>");
                    builder.Append("<Parameter name=\"resource\" />");
                    builder.Append("</Parameters>");
                    builder.Append("</Expression>");
                    builder.Append("</Constraint>");
                    builder.Append("</Constraints>");
                    // Define an anonymous type to define the possible constraint based group type code values.
                    var constraintBasedGroupTypeCode = new
                    {
                        Static = 0,
                        Dynamic = 1,
                        Implicit = 2
                    };
                    Logger.WriteDebugMessage("About to get User");
                    var systemUser = srv.SystemUserSet.FirstOrDefault(i => i.Id == PluginExecutionContext.InitiatingUserId);
                    
                    if (systemUser == null) return;
                    Logger.WriteDebugMessage("got user");

                    var group = new ConstraintBasedGroup
                    {
                        BusinessUnitId = systemUser.BusinessUnitId,
                        Constraints = builder.ToString(),
                        Name = "Selection Rule:" + McsHelper.getStringValue("cvt_name"),
                        GroupTypeCode = new OptionSetValue(constraintBasedGroupTypeCode.Implicit)
                    };

                    var newSysResource = OrganizationService.Create(group);
                    whereami = "created";
                    var spec = new ResourceSpec
                    {
                        BusinessUnitId = systemUser.BusinessUnitId,
                        ObjectiveExpression = @"<Expression><Body>udf ""Random""(factory,resource,appointment,request,leftoffset,rightoffset)</Body><Parameters><Parameter name=""factory"" /><Parameter name=""resource"" /><Parameter name=""appointment"" /><Parameter name=""request"" /><Parameter name=""leftoffset"" /><Parameter name=""rightoffset"" /></Parameters><Properties EvaluationInterval=""P0D"" evaluationcost=""0"" /></Expression>",
                        RequiredCount = 1,
                        Name = "Selection Rule:" + McsHelper.getStringValue("cvt_name"),
                        GroupObjectId = newSysResource,
                        SameSite = true
                    };
                    var _specId = OrganizationService.Create(spec);
                    Logger.setMethod = "Update Pat Group Resource";
                    Logger.WriteDebugMessage("About to update Patient Resource Group");
                    var updateResource = new Entity("cvt_patientresourcegroup") { Id = thisId };
                    updateResource["cvt_resourcespecguid"] = _specId.ToString();
                    updateResource["cvt_constraintgroupguid"] = newSysResource.ToString();
                    OrganizationService.Update(updateResource);
                    Logger.WriteDebugMessage("Patient Resource Group Updated");
                }
                catch (FaultException<OrganizationServiceFault> ex)
                {
                    Logger.WriteToFile(whereami + ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
                catch (Exception ex)
                {
                    Logger.WriteToFile(whereami + ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }
        #endregion

        #region Additional Interface Methods/Properties
        public override string McsSettingsDebugField
        {
            get { return "mcs_patresourcegroupplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }
        #endregion
    }
}