﻿using System.Linq;
using mcsScheduling;
using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using MCSShared;

namespace CVTPlugins
{
    /// <summary>
    /// Update the MTSA or TSA string fields since the Prov Group is being deleted
    /// </summary>
    public class CvtProvGroupDeletePreStageRunner : MCSPlugins.PluginRunner
    {
        #region Constructor
        public CvtProvGroupDeletePreStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods
        internal void Execute()
        {
            try
            {
                TracingService.Trace("at the top");
                Logger.setMethod = "Execute";
                //start the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Starting : {0}", GetType()));
                Logger.WriteDebugMessage(String.Format("Starting : {0}", GetType()));

                //checks to make sure the registration is correct
                if (!PluginExecutionContext.InputParameters.Contains("Target") || !(PluginExecutionContext.InputParameters["Target"] is Entity))
                    return;

                CvtHelper.UpdateMTSA(McsHelper.getEntRefID("cvt_RelatedMasterTSAId"), PluginExecutionContext.PrimaryEntityId, Logger, OrganizationService);
                CvtHelper.CreateUpdateService(McsHelper.getEntRefID("cvt_RelatedTSAid"), Logger, OrganizationService, McsSettings);

                Logger.WriteDebugMessage("Ending");
                //End the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Ending : {0}", GetType()));
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage);
            }
            catch (Exception ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage);
            }
        }
        #endregion

        #region Additional Interface Methods
        public override string McsSettingsDebugField
        {
            get { return "mcs_provresourcegroupplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.PreEntityImages["pre"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PreEntityImages["pre"];
        }
        #endregion
    }
}