﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MCSPlugins;
using Microsoft.Xrm.Sdk;
using System.ServiceModel;
using mcsScheduling;
using Microsoft.Xrm.Sdk.Query;
using MCSShared;
using System.Reflection;
using Microsoft.Xrm.Sdk.Messages;
using Microsoft.Crm.Sdk.Messages;

namespace CVTPlugins
{
    public class McsSystemSettingsCreatePostStageRunner : PluginRunner
    {
        public McsSystemSettingsCreatePostStageRunner(IServiceProvider serviceProvider) : base(serviceProvider) { }
        
        #region Primary Functionality
        /// <summary>
        /// Execute method is the entry point into the runner class, and is what is called by the Actual Plugin: McsSystemSettingsCreatePostStage
        /// </summary>
        /// <remarks>
        /// The plugin checks that the new System Settings Record is Called "Update Resources".  If so, it goes through and updates all Resources so that their IDs are exposed.  It also loops through all components and does the same thing.  
        /// </remarks>
        internal void Execute()
        {
            try
            {
                Logger.setMethod = "Execute";
                Logger.WriteTxnTimingMessage("Starting");

                if (PluginExecutionContext.InputParameters.Contains("Target") &&
                    PluginExecutionContext.InputParameters["Target"] is Entity)
                {
                    if (((Entity)PluginExecutionContext.InputParameters["Target"]).LogicalName != mcs_setting.EntityLogicalName)
                        return;
                    var entity = (Entity)PluginExecutionContext.InputParameters["Target"];
                    Logger.WriteDebugMessage(String.Format("At the top of:{0}", GetType()));
                    //Create the system resource
                    switch (entity.Attributes["mcs_name"].ToString().ToLower())
                    {
                        case "active settings":
                            break;
                        case "align visn":
                            UpdateFacilitySiteTSSResource();
                            goto case "update tssresources";
                        case "deploy":
                            //Sprint 31 //8/4/2015
                            //UpdateFacilitySiteTSSResource();

                            //Sprint 32
                            //Reset all debug fields to false:
                            //ResetPluginSettings();

                            //Update M/TSA with Provider Location Type data = Clinic Based
                            UpdateTSAData();
                            UpdateMTSAData();

                            goto case "delete";
                        case "setup schedulers":
                            Logger.WriteDebugMessage("Starting Set Up Schedulers");
                            setupSchedulingTeams();
                            Logger.WriteDebugMessage("Ending Setup Schedulers");
                            goto case "delete";
                        case "update resources":
                            Logger.WriteGranularTimingMessage("Starting UpdateResourceIDs");
                            UpdateResourceIDs();
                            Logger.WriteGranularTimingMessage("Ending UpdateResourceIDs");
                            UpdateComponentResourceIDs();
                            Logger.WriteGranularTimingMessage("Ending UpdateComponentReferenceIDs");
                            goto case "delete";
                        case "update time zones":
                            Logger.WriteGranularTimingMessage("Starting Update Time Zones");
                            UpdateUserTimeZones();
                            Logger.WriteGranularTimingMessage("Ending Update Time Zones");
                            goto case "delete";
                        case "update tsas":
                            Logger.WriteGranularTimingMessage("Starting Update TSA");
                            UpdateTSAData();
                            Logger.WriteGranularTimingMessage("Ending Update TSA");
                            Logger.WriteGranularTimingMessage("Starting Update MTSA");
                            UpdateMTSAData();
                            Logger.WriteGranularTimingMessage("Ending Update MTSA");
                            goto case "delete";
                        case "update tssresources":
                            Logger.WriteGranularTimingMessage("Starting UpdateTSSResourceData");
                            UpdateTSSResourceData();
                            Logger.WriteGranularTimingMessage("Ending UpdateTSSResourceData");
                            goto case "delete";
                        case "update teams":
                            Logger.WriteGranularTimingMessage("Starting UpdateTeamSecurityRoles");
                            UpdateTeamSecurityRoles();
                            Logger.WriteGranularTimingMessage("Ending UpdateTeamSecurityRoles");
                            goto case "delete";
                        case "delete":
                            DeletePluginSettingsRecord(entity);
                            break;
                        default:
                            var message = "The action you specified is not recognized, please check spelling or spacing.";
                            Logger.WriteDebugMessage(message);
                            throw new InvalidPluginExecutionException(message);
                    }
                }
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));

                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }

        internal void setupSchedulingTeams()
        {
            Logger.setMethod = "setupSchedulingTeams";
            Logger.WriteGranularTimingMessage("Starting setupSchedulingTeams");
            using (var srv = new Xrm(OrganizationService))
            {
                //LINQ Query to get all Users with TSS Scheduler Type
                var schedulerUsers = (from u in srv.SystemUserSet
                                      join ur in srv.SystemUserRolesSet on u.Id equals ur.SystemUserId.Value
                                      join r in srv.RoleSet on ur.RoleId equals r.RoleId
                                      where r.Name.Contains("TSS Scheduler") 
                                      where u.cvt_facility != null
                                      where u.IsDisabled.Value != true
                                      select new
                                      {
                                          u.Id,
                                          u.cvt_facility
                                      }).ToList();

                var facilities = srv.mcs_facilitySet.ToList();
                foreach (var fac in facilities)
                {
                    Logger.WriteDebugMessage("Beginning Loop for " + fac.mcs_name);
                    var thisFacilitySchedulers = new List<Guid>();

                    //Filter Scheduler type users from LINQ query to only use this facility
                    foreach (var user in schedulerUsers)
                    {
                        if (user.cvt_facility != null && user.cvt_facility.Id == fac.Id)
                            thisFacilitySchedulers.Add(user.Id);
                    }
                    Guid teamId = new Guid();
                    var existingTeam = srv.TeamSet.FirstOrDefault(t => t.cvt_Facility.Id == fac.Id && t.cvt_Type != null && t.cvt_Type.Value == 917290005);
                    if (existingTeam == null)
                        teamId = CreateSchedulerTeam(fac);
                    else
                        teamId = existingTeam.Id;

                    //If there are schedulers for this facility, then go ahead and add them to the Scheduler Team
                    if (thisFacilitySchedulers.Count > 0)
                    {
                        AddMembersTeamRequest amtr = new AddMembersTeamRequest()
                        {
                            TeamId = teamId,
                            MemberIds = thisFacilitySchedulers.ToArray(),
                        };
                        try
                        {
                            OrganizationService.Execute(amtr);
                            Logger.WriteDebugMessage("Team Members Added to " + fac.mcs_name);
                        }
                        catch (Exception ex)
                        {
                            Logger.WriteToFile("Unable to Add all team members, looping through individual team members: " + ex.Message);
                            foreach (var user in thisFacilitySchedulers)
                            {
                                try
                                {
                                    AddMembersTeamRequest req = new AddMembersTeamRequest()
                                    {
                                        TeamId = teamId,
                                        MemberIds = new Guid[1]{user}
                                    };
                                    OrganizationService.Execute(req);
                                    Logger.WriteDebugMessage("Added individual Member to Scheduler Team for " + fac.mcs_name);
                                }
                                catch (Exception ex2)
                                {
                                    Logger.WriteToFile(String.Format("Failed to add individual member {0} : {1}", user, ex2.Message));
                                }
                            }
                        }
                    }
                }
                Logger.WriteGranularTimingMessage("Finished setupSchedulingTeams");
                Logger.WriteDebugMessage("Scheduler Team Setup Complete");
            }
        }

        internal Guid CreateSchedulerTeam(mcs_facility facility)
        {
            var team = new Team()
            {
                Name = String.Format("Scheduler Team for {0} ({1})", facility.mcs_name, facility.mcs_StationNumber),
                cvt_Type = new OptionSetValue(917290005),
                BusinessUnitId = facility.mcs_BusinessUnitId,
                cvt_Facility = new EntityReference() { Id = facility.Id, Name = facility.mcs_name, LogicalName = facility.LogicalName }
            };
            return OrganizationService.Create(team);
        }

        internal void UpdateFacilitySiteTSSResource()
        {
            Logger.setMethod = "UpdateFacilitySiteTSSResource";
            Logger.WriteGranularTimingMessage("Starting UpdateFacilitySiteTSSResource");
            using (var srv = new Xrm(OrganizationService))
            {
                //Query for all Facilities
                var allFacilities = srv.mcs_facilitySet;

                foreach (var facility in allFacilities)
                {
                    CvtHelper.AlignLocations(facility, OrganizationService, Logger);
                }
            }
            Logger.setMethod = "UpdateFacilitySiteTSSResource";
            Logger.WriteGranularTimingMessage("Ending UpdateFacilitySiteTSSResource");
        }

        internal Team UpdateMTSAOwner(cvt_mastertsa mtsa, Xrm srv)
        {
            if (mtsa.OwningTeam == null || mtsa.OwnerId.Name == null || mtsa.OwnerId.Name.Contains("VISN"))
                return null;
            var fac = srv.mcs_facilitySet.FirstOrDefault(f => f.Id == mtsa.cvt_RelatedSiteid.Id);
            return srv.TeamSet.FirstOrDefault(t => t.Name == fac.mcs_BusinessUnitId.Name);
        }

        internal Team UpdateTSAOwner(mcs_services tsa, Xrm srv)
        {
            if (tsa.OwningTeam == null || tsa.OwnerId.Name == null || !tsa.OwnerId.Name.Contains("VISN"))
                return null;
            var fac = srv.mcs_facilitySet.FirstOrDefault(f => f.Id == tsa.cvt_ProviderFacility.Id);
            return srv.TeamSet.FirstOrDefault(t => t.Name == fac.mcs_BusinessUnitId.Name);
        }

        internal void ResetPluginSettings()
        {
            Logger.setMethod = "ResetPluginSettings";
            Logger.WriteDebugMessage("Starting ResetPluginSettings");
            //reset all of the booleans debug fields on the Plugin Settings - Active Settings record.
            using (var srv = new Xrm(OrganizationService))
            {
                var activeSettings = srv.mcs_settingSet.FirstOrDefault(s => s.mcs_name == "Active Settings");
                if (activeSettings != null)
                {
                    var updateSettings = new mcs_setting()
                     {
                         Id = activeSettings.Id
                     };
                    var update = false;

                    //set all of these debug fields to false
                    //foreach (KeyValuePair<String, Object> field in activeSettings.Attributes) {                        
                    //    //Might be Boolean, attempt to set it
                    //    if (activeSettings.Attributes[field.Key.ToString()].GetType().Name.ToString() == "Boolean")
                    //    {
                    //        if ((field.Value == null) || (Convert.ToBoolean(field.Value) == true))
                    //        {
                    //            updateSettings.Attributes[field.Key.ToString()] = false;
                    //            update = true;
                    //        }
                    //    }
                    //}

                    Type type = activeSettings.GetType();
                    PropertyInfo[] properties = type.GetProperties();

                    foreach (PropertyInfo property in properties)
                    {
                        //Might be Boolean, attempt to set it
                        object value = property.GetValue(activeSettings, null);
                        if (value != null)
                        {
                            Type propType = value.GetType();
                            if (propType == typeof(Boolean))
                            {
                                if (Convert.ToBoolean(property.GetValue(activeSettings, null)) == true)
                                {
                                    updateSettings.Attributes[property.Name] = false;
                                    update = true;
                                }
                            }
                        }
                        else //need to check the property against the attributes
                        {
                            if (activeSettings.Attributes[property.Name].GetType().Name.ToString() == "Boolean") 
                            {
                                updateSettings.Attributes[property.Name] = false;
                                update = true;
                            }
                        }
                    }
                    if (update == true)
                    {
                        Logger.WriteDebugMessage("Reset Active Settings record");
                        OrganizationService.Update(updateSettings);
                    }
                }
            }
            Logger.WriteDebugMessage("Ending ResetPluginSettings");
        }
        /// <summary>
        /// Update all user option time zones based on user records
        /// </summary>
        internal void UpdateUserTimeZones()
        {
            Logger.setMethod = "UpdateUserTimeZones";
            Logger.WriteDebugMessage("Update User Time Zones Begin");
            using (var srv = new Xrm(OrganizationService))
            {

                                    //var orphService = from service in srv.ServiceSet
                                    //  join tsa in srv.mcs_servicesSet on service.Id equals tsa.mcs_RelatedServiceId.Id into JoinedSvcTsa
                                    //  from tsa in JoinedSvcTsa.DefaultIfEmpty()

                var users = (from u in srv.SystemUserSet
                            join s in srv.UserSettingsSet on u.Id equals s.SystemUserId.Value
                            select new 
                             {
                                 u.Id,
                                 u.cvt_TimeZone,
                                 s.TimeZoneCode
                             });
                var numberFound = 0;
                var numberUpdated = 0;
                foreach (var user in users)
                {
                    numberFound++;
                    if (user.cvt_TimeZone != user.TimeZoneCode && user.cvt_TimeZone != null)
                    {
                        var updatedSetting = new UserSettings()
                        {
                            Id = user.Id,
                            TimeZoneCode = user.cvt_TimeZone
                        };
                        OrganizationService.Update(updatedSetting);
                        numberUpdated++;
                    }
                    else
                    {


                    }
                }
                Logger.WriteGranularTimingMessage("Found " + numberFound.ToString() + " Resources");
                Logger.WriteGranularTimingMessage("Updated " + numberUpdated.ToString() + " Resources");
            }
            Logger.WriteDebugMessage("Update User Time Zones End");
        }

        /// <summary>
        /// This utility method queries for all Resources, then copies their system generated ID into an exposed field that can be searched and used by regular users
        /// </summary>
        internal void UpdateResourceIDs()
        {
            using(var srv = new Xrm(OrganizationService))
            {
                var resources = (from r in srv.mcs_resourceSet
                                 where (r.cvt_systemtype.Value == 1)
                                 select new
                                 {
                                     r.Id,
                                     r.cvt_Identifier
                                 });
                var numberFound = 0;
                var numberUpdated = 0;
                foreach (var resource in resources)
                {
                    numberFound++;
                    if (resource.cvt_Identifier == null)
                    {
                        var update = new mcs_resource()
                        {
                            cvt_Identifier = resource.Id.ToString(),
                            Id = resource.Id
                        };
                        OrganizationService.Update(update);
                        numberUpdated++;
                    }
                }
                Logger.WriteGranularTimingMessage("Found " + numberFound.ToString() + " Resources");
                Logger.WriteGranularTimingMessage("Updated " + numberUpdated.ToString() + " Resources");
            }
        }

        /// <summary>
        /// This method updates component record with the resource ID
        /// </summary>
        internal void UpdateComponentResourceIDs()
        {
            using (var srv = new Xrm(OrganizationService))
            {
                var components = srv.cvt_componentSet;
                var numberUpdated = 0;
                var numberFound = 0;
                foreach (var component in components)
                {
                    numberFound++;
                    if (component.cvt_ParentResourceIdentifier != null)
                    {
                        var update = new cvt_component()
                        {
                            Id = component.Id,
                            cvt_ParentResourceIdentifier = component.cvt_relatedresourceid.Id.ToString()
                        };
                        OrganizationService.Update(update);
                        numberUpdated++;
                    }
                }
                Logger.WriteGranularTimingMessage("Found " + numberFound.ToString() + " Components");
                Logger.WriteGranularTimingMessage("Updated " + numberUpdated.ToString() + " Components");
            }
        }

        /// <summary>
        /// This method updates all TSAs - Type, Modality, Facility Patient and Provider
        /// </summary>
        internal void UpdateTSAData()
        {
            Logger.setMethod = "UpdateTSAData";
            Logger.WriteDebugMessage("starting UpdateTSAData");

            try
            {

                using (var srv = new Xrm(OrganizationService))
                {
                    var tsas = srv.mcs_servicesSet.ToList();
                    var numberFound = 0;
                    var numberUpdated = 0;
                    var servicesRebuilt = 0;
                    Logger.WriteDebugMessage("TSAs Found " + tsas.Count());

                    foreach (var tsa in tsas)
                    {
                        numberFound++;
                        if (numberFound.ToString().EndsWith("00"))
                            Logger.WriteDebugMessage("Starting Loop " + numberFound);
                        try
                        {
                            Guid patientFacility = (tsa.cvt_PatientFacility != null) ? tsa.cvt_PatientFacility.Id : Guid.Empty;
                            Guid providerFacility = (tsa.cvt_ProviderFacility != null) ? tsa.cvt_ProviderFacility.Id : Guid.Empty;
                            var updateTSA = false;
                            var update = new mcs_services()
                            {
                                Id = tsa.Id
                            };


                            //Provider Location Type: update all to Clinic Based (false)
                            if (tsa.cvt_ProviderLocationType == null)
                            {
                                update.cvt_ProviderLocationType = false;
                                updateTSA = true;
                            }

                            //Patient Location Type: Update null to false
                            if (tsa.cvt_Type == null)
                            {
                                update.cvt_Type = false;
                                updateTSA = true;
                            }

                            //Modality: Update null to false
                            if (tsa.cvt_AvailableTelehealthModalities == null)
                            {
                                update.cvt_AvailableTelehealthModalities = new OptionSetValue(917290000);
                                updateTSA = true;
                            }

                            //Patient Facility
                            if (patientFacility == Guid.Empty) //must be Single TSA or Home/Mobile
                            {
                                if (tsa.cvt_relatedpatientsiteid != null)
                                {
                                    //Get Facility of Patient Site
                                    mcs_site patSite = (mcs_site)OrganizationService.Retrieve(mcs_site.EntityLogicalName, tsa.cvt_relatedpatientsiteid.Id, new ColumnSet(true));

                                    //Set Facility to TSA
                                    if (patSite.mcs_FacilityId != null)  //Should always be filled
                                    {
                                        update.cvt_PatientFacility = patSite.mcs_FacilityId;
                                        updateTSA = true;
                                        patientFacility = patSite.mcs_FacilityId.Id;
                                    }
                                }
                            }

                            //Provider Facility
                            if (providerFacility == Guid.Empty)
                            {
                                if (tsa.cvt_relatedprovidersiteid != null) //Should always be filled
                                {
                                    //Get Facility of Provider Site
                                    mcs_site proSite = (mcs_site)OrganizationService.Retrieve(mcs_site.EntityLogicalName, tsa.cvt_relatedprovidersiteid.Id, new ColumnSet(true));

                                    //Set Facility to TSA
                                    if (proSite.mcs_FacilityId != null)  //Should always be filled
                                    {
                                        update.cvt_ProviderFacility = proSite.mcs_FacilityId;
                                        updateTSA = true;
                                        providerFacility = proSite.mcs_FacilityId.Id;
                                    }
                                }
                            }

                            //Service Scope: Inter (diff) vs Intra (same)
                            if ((patientFacility == providerFacility) || (patientFacility == Guid.Empty)) //Set TSA to IntraFacility
                            {
                                if (tsa.cvt_ServiceScope != null) //has value
                                {
                                    if (tsa.cvt_ServiceScope.Value != 917290001)
                                    {
                                        update.cvt_ServiceScope = new OptionSetValue(917290001);
                                        updateTSA = true;
                                    }
                                }
                                else
                                {
                                    update.cvt_ServiceScope = new OptionSetValue(917290001);
                                    updateTSA = true;
                                }
                            }
                            else //Set TSA to InterFacility
                            {
                                if (tsa.cvt_ServiceScope != null) //has value
                                {
                                    if (tsa.cvt_ServiceScope.Value != 917290000)
                                    {
                                        update.cvt_ServiceScope = new OptionSetValue(917290000);
                                        updateTSA = true;
                                    }
                                }
                                else
                                {
                                    update.cvt_ServiceScope = new OptionSetValue(917290000);
                                    updateTSA = true;
                                }
                            }

                            var team = UpdateTSAOwner(tsa, srv);
                            if (team != null && tsa.OwningTeam.Id != team.Id)
                            {
                                var ar = new AssignRequest()
                                {
                                    Assignee = new EntityReference() { Id = team.Id, Name = team.Name, LogicalName = Team.EntityLogicalName },
                                    Target = new EntityReference() { Id = tsa.Id, Name = tsa.mcs_name, LogicalName = mcs_services.EntityLogicalName }
                                };
                                OrganizationService.Execute(ar);
                            }

                            //Check to update TSA
                            if (updateTSA == true)
                            {
                                OrganizationService.Update(update);
                                numberUpdated++;
                            }

                            //Query the service if the TSA is in production
                            if (tsa.statuscode.Value == 251920000) //Production
                            {
                                try
                                {
                                    var tsaService = srv.ServiceSet.FirstOrDefault(s => s.Id == tsa.mcs_RelatedServiceId.Id);
                                    if (tsaService != null)
                                    {
                                        if (tsaService.Duration.Value != tsa.cvt_duration.Value)
                                        //Rebuild TSA Service
                                        {
                                            //CvtHelper.CreateUpdateService(tsa.Id, Logger, OrganizationService, McsSettings);
                                            var service = new Service()
                                            {
                                                Id = tsaService.Id,
                                                Duration = tsa.cvt_duration
                                            };
                                            OrganizationService.Update(service);
                                            servicesRebuilt += 1;
                                        }
                                    }
                                }
                                catch (Exception err)
                                {
                                    Logger.WriteDebugMessage("Errored when attempting to rebuild the service for this TSA: " + tsa.mcs_name + "Error: " + err);
                                }
                            }

                        }
                        catch (Exception ex)
                        {
                            Logger.WriteDebugMessage(String.Format("Unable to update TSA: {0} {1} due to exception: {2}", tsa.mcs_name, tsa.Id, ex.Message));
                        }
                    }
                    Logger.WriteDebugMessage("Found " + numberFound.ToString() + " TSAs");
                    Logger.WriteDebugMessage("Updated " + numberUpdated.ToString() + " TSAs");
                    Logger.WriteDebugMessage("Rebuilt " + servicesRebuilt.ToString() + " Services (Duration modified)");
                }
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }

        /// <summary>
        /// This method updates all MTSAs - Provider Facility, Type, Modality
        /// </summary>
        internal void UpdateMTSAData()
        {
            Logger.setMethod = "UpdateMTSAData";
            Logger.WriteDebugMessage("starting UpdateMTSAData");

            using (var srv = new Xrm(OrganizationService))
            {
                var mtsas = srv.cvt_mastertsaSet.ToList();
                var numberFound = 0;
                var numberUpdated = 0;
                Logger.WriteDebugMessage("Master TSAs Found: " + mtsas.Count());
                foreach (var mtsa in mtsas)
                {
                    try
                    {
                        numberFound++;
                        if (numberFound.ToString().EndsWith("00"))
                            Logger.WriteDebugMessage("Starting Loop " + numberFound);

                        Guid providerFacility = (mtsa.cvt_ProviderFacility != null) ? mtsa.cvt_ProviderFacility.Id : Guid.Empty;
                        var updateMTSA = false;
                        var update = new cvt_mastertsa()
                        {
                            Id = mtsa.Id
                        };

                        //if return is null, then use initial. Update all Initial Appointment Length to null. 
                        if (mtsa.cvt_InitialPatientAppointmentLength != null)
                        {
                            if (mtsa.cvt_ReturningPatientAppointmentLength == null)
                            {
                                update.cvt_ReturningPatientAppointmentLength = mtsa.cvt_InitialPatientAppointmentLength;
                            }
                            update.cvt_InitialPatientAppointmentLength = null;
                            updateMTSA = true;
                        }

                        //Provider Location Type: update all to Clinic Based (false)
                        if (mtsa.cvt_ProviderLocationType == null)
                        {
                            update.cvt_ProviderLocationType = false;
                            updateMTSA = true;
                        }

                        //Patient Location Type: Update null to false
                        if (mtsa.cvt_Type == null)
                        {
                            update.cvt_Type = false;
                            updateMTSA = true;
                        }

                        //Modality: Update null to false
                        if (mtsa.cvt_AvailableTelehealthModalities == null)
                        {
                            update.cvt_AvailableTelehealthModalities = new OptionSetValue(917290000);
                            updateMTSA = true;
                        }

                        //Provider Facility
                        if (mtsa.cvt_ProviderFacility == null)
                        {
                            if (mtsa.cvt_RelatedSiteid != null) //Should always be filled
                            {
                                //Get Facility of Provider Site
                                mcs_site proSite = (mcs_site)OrganizationService.Retrieve(mcs_site.EntityLogicalName, mtsa.cvt_RelatedSiteid.Id, new ColumnSet(true));

                                //Set Facility to TSA
                                if (proSite.mcs_FacilityId != null)  //Should always be filled
                                {
                                    update.cvt_ProviderFacility = proSite.mcs_FacilityId;
                                    updateMTSA = true;
                                    providerFacility = proSite.mcs_FacilityId.Id;
                                }
                            }
                        }

                        var team = UpdateMTSAOwner(mtsa, srv);
                        if (team != null && mtsa.OwningTeam.Id != team.Id)
                        {
                            var ar = new AssignRequest()
                            {
                                Assignee = new EntityReference() { Id = team.Id, Name = team.Name, LogicalName = Team.EntityLogicalName },
                                Target = new EntityReference() { Id = mtsa.Id, Name = mtsa.cvt_name, LogicalName = cvt_mastertsa.EntityLogicalName }
                            };
                            OrganizationService.Execute(ar);
                        }

                        //Check to update MTSA
                        if (updateMTSA == true)
                        {
                            OrganizationService.Update(update);
                            numberUpdated++;
                        }
                    }
                    catch (FaultException<OrganizationServiceFault> ex)
                    {
                        Logger.WriteToFile(ex.Message);
                        throw new InvalidPluginExecutionException(ex.Message);
                    }
                    catch (Exception ex)
                    {
                        if (ex.Message.StartsWith("custom"))
                        {
                            Logger.WriteDebugMessage(ex.Message.Substring(6) + mtsa.cvt_name);
                            throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                        }
                        else
                        {
                            Logger.WriteToFile(ex.Message + mtsa.cvt_name);
                            throw new InvalidPluginExecutionException(ex.Message);
                        }
                    }
                }
                Logger.WriteDebugMessage("Found " + numberFound.ToString() + " MTSAs");
                Logger.WriteDebugMessage("Updated " + numberUpdated.ToString() + " MTSAs");
                Logger.WriteDebugMessage("ending UpdateMTSAData");
            }
        }
        internal void UpdateTeamSecurityRoles()
        {
            Logger.setMethod = "UpdateTeamSecurityRoles";
            Logger.WriteDebugMessage("starting UpdateTeamSecurityRoles");

            using (var srv = new Xrm(OrganizationService))
            {
                var teams = srv.TeamSet.Where(t=>t.cvt_Type != null).ToList();
                var numberAdded = 0;
                var numberRemoved = 0;
                var numberUpdated = 0;
                Logger.WriteDebugMessage("Teams Found: " + teams.Count());
                foreach (var team in teams)
                {
                    try
                    {
                        int addedRoles = 0;
                        int removedRoles = 0;

                        CvtHelper.UpdateSecurityRoles(team.Id, Team.EntityLogicalName, OrganizationService, Logger, out addedRoles, out removedRoles);
                        if (removedRoles + addedRoles > 0)
                        {
                            Logger.WriteDebugMessage(String.Format("Team Security Role maintenance: {0}, Added {1}, Removed {2}", team.Name, addedRoles, removedRoles));
                            numberUpdated++;

                            if (removedRoles > 0)
                                numberRemoved++;
                            if (addedRoles > 0)
                                numberAdded++;
                        }
                    }

                    catch (FaultException<OrganizationServiceFault> ex)
                    {
                        Logger.WriteToFile(ex.Message);
                        throw new InvalidPluginExecutionException(ex.Message);
                    }
                    catch (Exception ex)
                    {
                        if (ex.Message.StartsWith("custom"))
                        {
                            Logger.WriteDebugMessage(ex.Message.Substring(6) + team.Name);
                            throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                        }
                        else
                        {
                            Logger.WriteToFile(ex.Message + team.Name);
                            throw new InvalidPluginExecutionException(ex.Message);
                        }
                    }
                }
                Logger.WriteDebugMessage("Found " + teams.Count().ToString() + " Teams");
                Logger.WriteDebugMessage("Number of Teams Updated " + numberUpdated.ToString());
                Logger.WriteDebugMessage("Added roles to " + numberAdded.ToString() + " Teams");
                Logger.WriteDebugMessage("Removed roles from " + numberRemoved.ToString() + " Teams");

                Logger.WriteDebugMessage("ending UpdateTeamSecurityRoles");
            }
        }
        ////Clear all unused services - probably resource intensive
        ////Need to check for any service activities, it would prevent a delete.
        //public static int DeleteUnusedServices(IOrganizationService OrganizationService)
        //{
        //    int i = 0;
        //    using (var srv = new Xrm(OrganizationService))
        //    {
        //        try
        //        {
        //            /*All Services*/
        //            //SERVICE LEFT JOIN TSAS on service.id = TSA.relatedserviceid
        //            var orphService = from service in srv.ServiceSet
        //                              join tsa in srv.mcs_servicesSet on service.Id equals tsa.mcs_RelatedServiceId.Id into JoinedSvcTsa
        //                              from tsa in JoinedSvcTsa.DefaultIfEmpty()
        //                              select new
        //                              {
        //                                  ServiceId = service.Id,
        //                                  TSAId = tsa != null ? tsa.mcs_RelatedServiceId : null
        //                              };

        //            foreach (var result in orphService)
        //            {
        //                if (result.TSAId == null)
        //                {
        //                    //Search on the prior service and attempt to delete it.
        //                    var ServiceActivity = srv.ServiceAppointmentSet.FirstOrDefault(j => j.ServiceId.Id == result.ServiceId);
        //                    //If an associated SA doesn't exist, delete
        //                    if (ServiceActivity == null)
        //                    {
        //                        OrganizationService.Delete(Service.EntityLogicalName, result.ServiceId);
        //                        i += 1;
        //                    }
        //                    else
        //                    {
        //                        //Deactivate
        //                    }
        //                }
        //            }
        //        }
        //        catch (Exception ex)
        //        {
        //            Logger.WriteToFile(ex.Message);
        //        }
        //        return i; //Number of records deleted
        //    }
        //}

        /// <summary>
        /// Update Facility and VISN on all TSS Resources
        /// </summary>
        internal void UpdateTSSResourceData()
        {
            using (var srv = new Xrm(OrganizationService))
            {
                var resources = srv.mcs_resourceSet;
                var numberFound = 0;
                var numberUpdated = 0;

                foreach (var resource in resources)
                {
                    numberFound++;
                    if (resource.mcs_RelatedSiteId != null) //Site is not null
                    {
                        //Get Facility of Site
                        mcs_site resourceSite = (mcs_site)OrganizationService.Retrieve(mcs_site.EntityLogicalName, resource.mcs_RelatedSiteId.Id, new ColumnSet(true));
                        Guid resourceFacility = (resource.mcs_Facility != null) ? resource.mcs_Facility.Id : Guid.Empty;
                        Guid siteFacility = (resourceSite.mcs_FacilityId != null) ? resourceSite.mcs_FacilityId.Id : Guid.Empty;
                        Guid resourceVISN = (resource.mcs_BusinessUnitId != null) ? resource.mcs_BusinessUnitId.Id : Guid.Empty;
                        Guid siteVISN = (resourceSite.mcs_BusinessUnitId != null) ? resourceSite.mcs_BusinessUnitId.Id : Guid.Empty;
                        var TSSResourceUpdate = false;
                        var update = new mcs_resource();

                        //check if need to update facility
                        if (resourceFacility != siteFacility) 
                        {
                            update.mcs_Facility = resourceSite.mcs_FacilityId;
                            TSSResourceUpdate = true;
                        }

                        //check if need to update visn
                        if (resourceVISN != siteVISN)
                        {
                            update.mcs_BusinessUnitId = resourceSite.mcs_BusinessUnitId;
                            TSSResourceUpdate = true;
                        }
                        
                        if (TSSResourceUpdate == true)
                        {
                            update.Id = resource.Id;
                            OrganizationService.Update(update);
                            numberUpdated++;
                        }
                    }
                }
                Logger.WriteDebugMessage("Found " + numberFound.ToString() + " TSS Resources");
                Logger.WriteDebugMessage("Updated " + numberUpdated.ToString() + " TSS Resources");
            }
        }

        /// <summary>
        /// Clean up the current record.
        /// </summary>
        /// <param name="entity"></param>
        internal void DeletePluginSettingsRecord(Entity entity)
        {
            using (var srv = new Xrm(OrganizationService))
            {
                Logger.setMethod = "DeletePluginSettingsRecord";
                OrganizationService.Delete(entity.LogicalName, entity.Id);
                Logger.WriteDebugMessage("Deleted Plugin Settings Record");
            }
        }
        #endregion

        #region AbstractClassRequiredMethods
        public override string McsSettingsDebugField
        {
            get { return "mcs_serviceplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}