﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MCSPlugins;
using Microsoft.Xrm.Sdk;
using System.ServiceModel;
using mcsScheduling;
using Microsoft.Xrm.Sdk.Query;
using MCSShared;
using System.Reflection;
using Microsoft.Xrm.Sdk.Messages;
using Microsoft.Crm.Sdk.Messages;

namespace CVTPlugins
{
    public class TeamCreatePostStageRunner : PluginRunner
    {
        public TeamCreatePostStageRunner(IServiceProvider serviceProvider) : base(serviceProvider) { }
        
        #region Primary Functionality
        /// <summary>
        /// Execute method is the entry point into the runner class, and is what is called by the Actual Plugin: TeamCreatePostStage
        /// </summary>
        /// <remarks>
        /// The plugin checks that the new Team Record gets the correctly assigned base security roles.
        /// </remarks>
        internal void Execute()
        {
            try
            {
                Logger.setMethod = "Execute";
                Logger.WriteTxnTimingMessage("Starting");

                if (PluginExecutionContext.InputParameters.Contains("Target") && PluginExecutionContext.InputParameters["Target"] is Entity)
                {
                    if (((Entity)PluginExecutionContext.InputParameters["Target"]).LogicalName != Team.EntityLogicalName)
                        return;
                    var entity = (Entity)PluginExecutionContext.InputParameters["Target"];
                    Logger.WriteDebugMessage(String.Format("At the top of:{0}", GetType()));

                    //Add the Security Roles:
                    int addedRoles;
                    int removedRoles;
                    CvtHelper.UpdateSecurityRoles(PluginExecutionContext.PrimaryEntityId, Team.EntityLogicalName, OrganizationService, Logger, out addedRoles, out removedRoles);
                }
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }
        #endregion

        #region AbstractClassRequiredMethods
        public override string McsSettingsDebugField
        {
            get { return "cvt_teamplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}