﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MCSPlugins;
using Microsoft.Xrm.Sdk;
using System.ServiceModel;
using mcsScheduling;
using Microsoft.Xrm.Sdk.Query;
using MCSShared;
using System.Reflection;
using Microsoft.Xrm.Sdk.Messages;
using Microsoft.Crm.Sdk.Messages;

namespace CVTPlugins
{
    public class TeamCreatePreStageRunner : PluginRunner
    {
        public TeamCreatePreStageRunner(IServiceProvider serviceProvider) : base(serviceProvider) { }
        
        #region Primary Functionality
        /// <summary>
        /// Execute method is the entry point into the runner class, and is what is called by the Actual Plugin: TeamCreatePreStage
        /// </summary>
        /// <remarks>
        /// The plugin checks that the new Team Record gets the correctly assigned BU based on facility (if selected).
        /// </remarks>
        internal void Execute()
        {
            try
            {
                Logger.setMethod = "Execute";
                Logger.WriteTxnTimingMessage("Starting");

                if (PluginExecutionContext.InputParameters.Contains("Target") && PluginExecutionContext.InputParameters["Target"] is Entity)
                {
                    if (((Entity)PluginExecutionContext.InputParameters["Target"]).LogicalName != Team.EntityLogicalName)
                        return;
                    var entity = (Entity)PluginExecutionContext.InputParameters["Target"];
                    Logger.WriteDebugMessage(String.Format("At the top of:{0}", GetType()));

                    Team thisTeam = entity.ToEntity<Team>();
                    if (thisTeam.cvt_Facility != null)
                    {
                        using (var srv = new Xrm(OrganizationService))
                        {
                            var facility = srv.mcs_facilitySet.First(f => f.Id == thisTeam.cvt_Facility.Id);
                            if (facility.mcs_BusinessUnitId != null)
                            {
                                Logger.WriteDebugMessage("On create of team (" + thisTeam.Name + "), updated VISN to match listed Facility's VISN: " + facility.mcs_BusinessUnitId.Name);
                                entity.Attributes["businessunitid"] = facility.mcs_BusinessUnitId;
                            }
                        }
                    }
                }
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }
        #endregion

        #region AbstractClassRequiredMethods
        public override string McsSettingsDebugField
        {
            get { return "cvt_teamplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}