﻿using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using mcsScheduling;
using System.Linq;
using System.Collections;
using System.ComponentModel;
using System.Configuration;
using System.Drawing;
using System.Web;
using System.Xml;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using MCSUtilities2011;
using Microsoft.Crm.Sdk;
using Microsoft.Crm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Client;
using Microsoft.Xrm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Query;

namespace MCSPlugins
{
    /// <summary>
    /// Update a related Service Activity to Close or Canceled when this Appointment is Closed or Canceled.
    /// </summary>
    public class McsAppointmentCloseCancelPostStageRunner : PluginRunner
    {
        #region Constructor
        public McsAppointmentCloseCancelPostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods
        internal void Execute()
        {
            try
            {
                Logger.setMethod = "Execute";
                TracingService.Trace("at the top");
                //start the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Starting : {0}", GetType()));
                Logger.WriteDebugMessage(String.Format("Starting : {0}", GetType()));

                //checks to make sure the registration is correct
                if (PluginExecutionContext.PrimaryEntityId != null)
                {
                    CloseCancelServiceActivity(PluginExecutionContext.PrimaryEntityId);
                }             
                Logger.setMethod = "Execute";
                //End the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Ending : {0}", GetType()));
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage + ":" + ex.Message);
                }
            }
        }

        internal void CloseCancelServiceActivity(Guid thisApptId)
        {
            Logger.setMethod = "CloseCancelServiceActivity";
            Logger.WriteDebugMessage("starting Delete Appointment");

            using (var srv = new Xrm(OrganizationService))
            {
                Logger.WriteDebugMessage("Looking for related Service Activity.");
                var ServiceActivity = srv.ServiceAppointmentSet.FirstOrDefault(i => i.mcs_appointmentid.Id == thisApptId);
                var Appointment = srv.AppointmentSet.FirstOrDefault(i => i.Id == thisApptId);

                if (ServiceActivity == null)
                    return;
                Logger.WriteDebugMessage("Retrieved Service Activity: " + ServiceActivity.Subject);

                //Get the StatusCode of this Appt
                var StatusCode = Appointment.StatusCode.Value;

                //set req to the Service Activity
                SetStateRequest req = new SetStateRequest();
                req.EntityMoniker = new EntityReference("serviceappointment", ServiceActivity.Id);

                //Closed
                if (StatusCode == 3)
                {
                    req.State = new OptionSetValue(1);
                    req.Status = new OptionSetValue(8);
                    SetStateResponse reqResponse = (SetStateResponse)OrganizationService.Execute(req);
                    Logger.WriteDebugMessage("Service Activity Status updated to Closed");
                }
                //Canceled
                if (StatusCode == 4)
                {
                    req.State = new OptionSetValue(2);
                    req.Status = new OptionSetValue(9);
                    SetStateResponse reqResponse = (SetStateResponse)OrganizationService.Execute(req);
                    Logger.WriteDebugMessage("Service Activity Status updated to Canceled");
                }
            }
        }
        #endregion
        
        #region Additional Interface Methods
        public override string McsSettingsDebugField
        {
            get { return "mcs_appointmentplugin"; }
        }
        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }
        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}