﻿using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using mcsScheduling;
using System.Linq;
using System.Collections;
using System.ComponentModel;
using System.Configuration;
using System.Drawing;
using System.Web;
using System.Xml;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using MCSUtilities2011;
using Microsoft.Crm.Sdk;
using Microsoft.Crm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Client;
using Microsoft.Xrm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Query;

namespace MCSPlugins
{
    /// <summary>
    /// To Create a Service Activity when an Appointment is created.
    /// </summary>
    public class McsAppointmentCreatePostStageRunner : PluginRunner
    {
        #region Constructor
        public McsAppointmentCreatePostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods
        internal void Execute()
        {
            try
            {
                TracingService.Trace("at the top");
                Logger.setMethod = "Execute";
                //start the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Starting : {0}", GetType()));
                Logger.WriteDebugMessage(String.Format("Starting : {0}", GetType()));

                //checks to make sure the registration is correct
                if (!PluginExecutionContext.InputParameters.Contains("Target") || !(PluginExecutionContext.InputParameters["Target"] is Entity))
                    return;

                CreateServiceActivity(PluginExecutionContext.PrimaryEntityId);  
                
                Logger.setMethod = "Execute";
                //End the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Ending : {0}", GetType()));
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage + ":" + ex.Message);
                }
            }
        }

        internal void CreateServiceActivity(Guid thisApptId)
        {
            Logger.setMethod = "CreateServiceActivity";
            Logger.WriteDebugMessage("starting CreateServiceActivity");

            using (var srv = new Xrm(OrganizationService))
            {
                //Get this Appt
                var thisAppt = srv.AppointmentSet.FirstOrDefault(i => i.Id == thisApptId);

                if ((thisAppt.cvt_serviceactivityid == null) || (thisAppt.SeriesId == null))
                {
                    Logger.WriteDebugMessage("Exiting, no Service Activity present");
                    return;
                }

                //Get Service Activity
                var retrieveServiceActivity = srv.ServiceAppointmentSet.FirstOrDefault(i => i.Id == thisAppt.cvt_serviceactivityid.Id);

                //Check if SA actually exists
                if (retrieveServiceActivity == null)
                    return;
                Logger.WriteDebugMessage("starting Service Activity Recurrence, since SA present");
                //Remove All Resources from this Appointment
                var updateAppt = new Appointment()
                {
                    Id = thisAppt.Id,
                    RequiredAttendees = null
                };
                OrganizationService.Update(updateAppt);
                Logger.WriteDebugMessage("Removed Resources from Appointment");

                //Here we are defining the attributes to be carried over to the new Service Activities created. Each is created for every Appointment created. 
                ServiceAppointment addServiceActivity = new ServiceAppointment();
                Logger.WriteDebugMessage("Defining Service Activity Attributes");
                addServiceActivity.Subject = retrieveServiceActivity.Subject;
                addServiceActivity.cvt_openappointment = retrieveServiceActivity.cvt_openappointment;
                addServiceActivity.mcs_groupappointment = retrieveServiceActivity.mcs_groupappointment;
                addServiceActivity.mcs_capacity = retrieveServiceActivity.mcs_capacity;
                addServiceActivity.mcs_relatedtsa = retrieveServiceActivity.mcs_relatedtsa;
                addServiceActivity.mcs_relatedsite = retrieveServiceActivity.mcs_relatedsite;
                addServiceActivity.mcs_relatedprovidersite = retrieveServiceActivity.mcs_relatedprovidersite;
                addServiceActivity.mcs_servicetype = retrieveServiceActivity.mcs_servicetype;
                addServiceActivity.mcs_servicesubtype = retrieveServiceActivity.mcs_servicesubtype;
                addServiceActivity.ServiceId = retrieveServiceActivity.ServiceId;
                addServiceActivity.cvt_recurrence = true;
                addServiceActivity.mcs_appointmentid = new EntityReference(thisAppt.LogicalName, thisAppt.Id);
                addServiceActivity.cvt_recurringappointmentsmaster = new EntityReference(RecurringAppointmentMaster.EntityLogicalName, thisAppt.SeriesId.Value);

                //Set the properties to the Appointment's Start/End
                addServiceActivity.ScheduledStart = thisAppt.ScheduledStart;
                addServiceActivity.ScheduledEnd = thisAppt.ScheduledEnd;

                //Creating a new Activity Party List and pulling all the resources from base Service Appointment to be carried over to recurring Service Activities. 
                List<ActivityParty> res = new List<ActivityParty>();
                var resources = retrieveServiceActivity.Resources.ToList();

                if (resources.Count > 0)
                {
                    for (int i = 0; i < resources.Count; i++)
                    {
                        ActivityParty party = new ActivityParty();
                        EntityReference entRef = resources[i].PartyId;
                        party.PartyId = entRef;
                        res.Add(party);
                    }

                    if (res.Count > 0)
                        addServiceActivity.Resources = res;
                }

                ////Creating a new Activity Party List and pulling all the patients from base Service Appointment to be carried over to recurring Service Activities.
                //List<ActivityParty> patients = new List<ActivityParty>();
                //var masterPatientList = retrieveServiceActivity.Customers.ToList();

                //if (masterPatientList.Count > 0)
                //{
                //    for (int i = 0; i < masterPatientList.Count; i++)
                //    {
                //        ActivityParty party = new ActivityParty();
                //        EntityReference test = masterPatientList[i].PartyId;
                //        party.PartyId = test;
                //        patients.Add(party);
                //    }

                //    if (patients.Count > 0)
                //        addServiceActivity.Customers = patients;
                //}

                Logger.WriteDebugMessage("Creating the new Service Activity");

                // Use the Book request message.
                BookRequest book = new BookRequest
                {
                    Target = addServiceActivity
                };
                BookResponse booked = (BookResponse)OrganizationService.Execute(book);
                var ServiceApptId = booked.ValidationResult.ActivityId;

                // Verify that the Service Activity has been scheduled.
                if (ServiceApptId != Guid.Empty)
                {
                    Logger.WriteDebugMessage(String.Format("Succesfully booked {0}.", addServiceActivity.Subject));
                    SetStateRequest req = new SetStateRequest();
                    req.EntityMoniker = new EntityReference(ServiceAppointment.EntityLogicalName, ServiceApptId);
                    req.State = new OptionSetValue(3);//Scheduled
                    req.Status = new OptionSetValue(4); //Reserved
                    SetStateResponse reqResponse = (SetStateResponse)OrganizationService.Execute(req);
                    Logger.WriteDebugMessage("Service Activity Status updated to Scheduled");
                }
                else
                {
                    //Because a booking conflict was detected, this Service Activity will be set to an Open State with the Status as "Conflict".
                    addServiceActivity.cvt_showtimeas = new OptionSetValue(917290000); //Open: Requested (Should it be Conflict?)
                    //addServiceActivity.StateCode = new OptionSetValue(
                    addServiceActivity.StatusCode = new OptionSetValue(1); //Was Tentative, now Requested?
                    var newServiceActivityConflict = OrganizationService.Create(addServiceActivity);
                }
            }
        }
        #endregion

        #region Additional Interface Methods
        public override string McsSettingsDebugField
        {
            get { return "mcs_appointmentplugin"; }
        }
        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }
        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}