﻿using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using mcsScheduling;
using System.Linq;
using System.Collections;
using System.ComponentModel;
using System.Configuration;
using System.Drawing;
using System.Web;
using System.Xml;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using MCSUtilities2011;
using Microsoft.Crm.Sdk;
using Microsoft.Crm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Client;
using Microsoft.Xrm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Query;

namespace MCSPlugins
{
    /// <summary>
    /// Delete a related Service Activities when this Appointment is being deleted.
    /// </summary>
    public class McsAppointmentDeletePreStageRunner : PluginRunner
    {
        #region Constructor
        public McsAppointmentDeletePreStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion
        
        #region Internal Methods
        internal void Execute()
        {
            try
            {
                TracingService.Trace("at the top");
                Logger.setMethod = "Execute";
                //start the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Starting : {0}", GetType()));
                Logger.WriteDebugMessage(String.Format("Starting : {0}", GetType()));

                //checks to make sure the registration is correct
                if (!PluginExecutionContext.InputParameters.Contains("Target") || !(PluginExecutionContext.InputParameters["Target"] is Entity))
                    return;
                               
                DeleteServiceActivity(PluginExecutionContext.PrimaryEntityId);                     

                Logger.WriteDebugMessage("Ending");
                //End the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Ending : {0}", GetType()));
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage + ":" + ex.Message);
                }
            }
        }
       
        internal void DeleteServiceActivity(Guid thisApptId)
        {
            Logger.setMethod = "DeleteServiceActivity";
            Logger.WriteDebugMessage("starting DeleteServiceActivity");                     

            using (var srv = new Xrm(OrganizationService))
            {            
                //Look for related Service Activities
                var ServiceActivity = srv.ServiceAppointmentSet.FirstOrDefault(i => i.mcs_appointmentid.Id == thisApptId);
                if (ServiceActivity == null)
                    return;
                Logger.WriteDebugMessage("Retrieved Service Activity: " + ServiceActivity.Subject);

                //Delete the related Service Activity
                OrganizationService.Delete(ServiceActivity.LogicalName, ServiceActivity.Id);
                Logger.WriteDebugMessage("Service Activity Deleted");
            }              
        }
        #endregion
        
        #region Additional Interface Methods      
        public override string McsSettingsDebugField
        {
            get { return "mcs_appointmentplugin"; }
        }
        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.PreEntityImages["pre"];
        }
        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PreEntityImages["pre"];
        }
        #endregion
    }
}