﻿using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using mcsScheduling;
using System.Linq;
using System.Collections;
using System.ComponentModel;
using System.Configuration;
using System.Drawing;
using System.Web;
using System.Xml;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using MCSUtilities2011;
using Microsoft.Crm.Sdk;
using Microsoft.Crm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Client;
using Microsoft.Xrm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Query;

namespace MCSPlugins
{
    /// <summary>
    /// Delete SAs if status is Close or Cancel on the Appt series.
    /// </summary>
    public class McsAppointmentMasterCloseCancelPostStageRunner : PluginRunner
    {
        #region Constructor
        public McsAppointmentMasterCloseCancelPostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods
        internal void Execute()
        {
            try
            {
                TracingService.Trace("at the top");
                Logger.setMethod = "Execute";
                //start the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Starting : {0}", GetType()));
                Logger.WriteDebugMessage(String.Format("Starting : {0}", GetType()));

                //checks to make sure the registration is correct
                if (PluginExecutionContext.PrimaryEntityId != null)
                {                    
                    CloseCancelAppointmentSeries(PluginExecutionContext.PrimaryEntityId);
                }             
                Logger.setMethod = "Execute";
                //End the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Ending : {0}", GetType()));
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage + ":" + ex.Message);
                }
            }
        }

        internal void CloseCancelAppointmentSeries(Guid thisApptMasterId)
        {
            Logger.setMethod = "CloseCancelAppointmentSeries";
            Logger.WriteDebugMessage("starting CloseCancelAppointmentSeries");
            try
            {
                using (var srv = new Xrm(OrganizationService))
                {
                    var RecurringMaster = srv.RecurringAppointmentMasterSet.FirstOrDefault(i => i.Id == thisApptMasterId);
                    // DateTime RecurringSeriesEndDate = RecurringMaster.date.Value;
                    var RecurringMasterStatusCode = RecurringMaster.StatusCode.Value;

                    //RecurringMaster Closed(3) or Cancelled(4)
                    if ((RecurringMasterStatusCode == 3) || (RecurringMasterStatusCode == 4))
                    {
                        //Check if the SA is Reserved(4) or Requested(1).
                        //Also check if the Service Activity is part of series that was ended early. We will delete the SA's that were schedueld outside the end date.
                        var ServiceActivities = from sa in srv.ServiceAppointmentSet
                                                where (sa.cvt_recurringappointmentsmaster.Id == thisApptMasterId)
                                                   && ((sa.StatusCode.Value == 1) || (sa.StatusCode.Value == 4))
                                                   && (sa.mcs_appointmentid == null)
                                                // && (sa.ScheduledStart.Value.CompareTo(RecurringSeriesEndDate) >= 0)
                                                && sa.StateCode == 0
                                                select sa;

                        var SACount = 0;
                        foreach (var sa in ServiceActivities)
                        {
                            SACount += 1;
                        }
                        if (SACount > 0)
                        {
                            Logger.WriteDebugMessage("Found " + SACount + " Service Activities. Starting Delete.");
                            foreach (var sa in ServiceActivities)
                            {
                                Logger.WriteDebugMessage("Got ServiceActivity:" + sa.Subject);
                                OrganizationService.Delete(sa.LogicalName, sa.Id);
                                Logger.WriteDebugMessage("Service Activity Deleted");
                            }
                        }
                    }
                }
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage + ":" + ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage + ":" + ex.Message);
                }
            }
        }       
        #endregion

        #region Additional Interface Methods
        public override string McsSettingsDebugField
        {
            get { return "mcs_appointmentplugin"; }
        }
        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }
        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}