﻿using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using mcsScheduling;
using System.Linq;
using System.Collections;
using System.ComponentModel;
using System.Configuration;
using System.Drawing;
using System.Web;
using System.Xml;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using MCSUtilities2011;
using Microsoft.Crm.Sdk;
using Microsoft.Crm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Client;
using Microsoft.Xrm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Query;

namespace MCSPlugins
{
    /// <summary>
    /// If occurrences > 16, throw custom error to user.
    /// </summary>
    public class McsAppointmentMasterCreatePreStageRunner : PluginRunner
    {
        #region Constructor
        public McsAppointmentMasterCreatePreStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion
        #region Internal Methods
        internal void Execute()
        {
            try
            {
                Logger.setMethod = "Execute";
                TracingService.Trace("at the top");

                //start the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Starting : {0}", GetType()));
                Logger.WriteDebugMessage("Starting");

                Logger.WriteDebugMessage(String.Format("At the top of: {0}", GetType()));

                //checks to make sure the registration is correct
                if (!PluginExecutionContext.InputParameters.Contains("Target") ||
                    !(PluginExecutionContext.InputParameters["Target"] is Entity))
                    return;

                var entity = (Entity)PluginExecutionContext.InputParameters["Target"];
                var thisRecurringSeries = entity.ToEntity<RecurringAppointmentMaster>();
                CheckAttributes(thisRecurringSeries);

                Logger.setMethod = "Execute";
                //End the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Ending : {0}", GetType()));
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(McsSettings.getUnexpectedErrorMessage + ":" + ex.Message);
                }
            }
        }

        internal void CheckAttributes(Entity thisRecurringSeries)
        {
            Logger.setMethod = "CheckAttributes";
            Logger.WriteDebugMessage("Checking Recurring Series Attributes");

            using (var srv = new Xrm(OrganizationService))
            {
                var occurrences = McsHelper.getIntValue("occurrences");

                if (occurrences > 16)
                {
                    Logger.WriteDebugMessage("Too many occurrences");
                    throw new InvalidPluginExecutionException("customNumber of Occurrences cannot exceed 16 ");
                }
            }
        }
        #endregion
        #region Additional Interface Methods
        public override string McsSettingsDebugField
        {
            get { return "mcs_appointmentplugin"; }
        }
        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }
        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }
        #endregion
    }
}