﻿using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using mcsScheduling;
using System.Linq;

namespace MCSPlugins
{
    public class McsGroupResourceCreatePostStageRunner : PluginRunner
    {
        //Purpose:  The purpose of this plugin is to create the system table entries needed for services to work - As these records are created the system resource group is updated

        #region Constructor
        public McsGroupResourceCreatePostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods/Properties
        internal void Execute()
        {
            try
            {
                Logger.setMethod = "Execute";

                //start the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Starting : {0}", GetType()));

                //checks to make sure the registration is correct
                if (PluginExecutionContext.InputParameters.Contains("Target") &&
                    PluginExecutionContext.InputParameters["Target"] is Entity)
                {

                    Logger.WriteDebugMessage(String.Format("At the top of:{0}", GetType()));

                    //start the timing for the submethod
                    Logger.WriteGranularTimingMessage("Starting UpdateResourceGroup");

                    UpdateGroupResource(PluginExecutionContext.PrimaryEntityId);
                    //update the resource group
                    UpdateResourceGroup(McsHelper.getEntRefID("mcs_relatedresourcegroupid"));

                    //end the timing for the submethod
                    Logger.WriteGranularTimingMessage("Ending UpdateResourceGroup");
                }

                Logger.setMethod = "Execute";

                //end the timing for the submethod


                Logger.WriteDebugMessage("Ending");
                //End the timing for the plugin
                Logger.WriteTxnTimingMessage(String.Format("Ending : {0}", GetType()));

            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));

                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }

        internal void UpdateGroupResource(Guid thisId)
        {
            Logger.setMethod = "Update Group Resource";
            Logger.WriteDebugMessage("starting UpdateGroupResource");

            using (var srv = new Xrm(OrganizationService))
            {
                try
                {
                    var thisGroupResource = srv.mcs_groupresourceSet.FirstOrDefault(i => i.Id == thisId);                  
                    var thisGroupResourceType = thisGroupResource.mcs_Type.Value;

                    if (thisGroupResourceType == 251920001)
                    {
                        var relatedResource = srv.mcs_resourceSet.FirstOrDefault(i => i.Id == thisGroupResource.mcs_RelatedResourceId.Id);
                        var updateGroupResource = new mcs_groupresource();
                        updateGroupResource.Id = thisId;
                        updateGroupResource.cvt_capacity = relatedResource.cvt_capacity;
                        OrganizationService.Update(updateGroupResource);
                    }
                }

                catch (FaultException<OrganizationServiceFault> ex)
                {
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
                catch (Exception ex)
                {
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }

        internal void UpdateResourceGroup(Guid thisId)
        {
            Logger.WriteDebugMessage("starting UpdateResourceGroup");

            var whereami = "top";

            using (var srv = new Xrm(OrganizationService))
            {
                try
                {
                    Logger.setMethod = "Update System Group Resource";

                    var RG = srv.mcs_resourcegroupSet.FirstOrDefault(i => i.Id == thisId);

                    var RGType = RG.mcs_Type.Value;

                    if ((RGType == 251920000) || (RGType == 251920001) || (RGType == 251920002))
                    {

                        var builder = new System.Text.StringBuilder("<Constraints>");
                        builder.Append("<Constraint>");
                        builder.Append("<Expression>");
                        var getResources = from resGroups in srv.mcs_groupresourceSet
                                           join mcsResourcs in srv.mcs_resourceSet on resGroups.mcs_RelatedResourceId.Id equals mcsResourcs.mcs_resourceId.Value
                                           where resGroups.mcs_relatedResourceGroupId.Id == thisId && resGroups.statecode == 0
                                           select new
                                           {
                                               mcsResourcs.mcs_name,
                                               mcsResourcs.mcs_relatedResourceId
                                           };

                        string resources = null; 
                        var count = 0;
                        foreach (var mcsResourcs in getResources)
                        {
                            //First add the resources to the text field for the view. 
                            resources += mcsResourcs.mcs_relatedResourceId.Name.ToString();
                            resources += " ; ";

                            builder.Append(count > 0 ? " || resource[\"Id\"] == " : "<Body>resource[\"Id\"] == ");
                            builder.Append(mcsResourcs.mcs_relatedResourceId.Id.ToString("B"));
                            count += 1;
                        }
                        if (count == 0)
                        {
                            builder.Append("<Body>false");

                        }

                        builder.Append("</Body>");
                        builder.Append("<Parameters>");
                        builder.Append("<Parameter name=\"resource\" />");
                        builder.Append("</Parameters>");
                        builder.Append("</Expression>");
                        builder.Append("</Constraint>");
                        builder.Append("</Constraints>");

                        var RelatedConstraintRG = srv.ConstraintBasedGroupSet.FirstOrDefault(i => i.Id == RG.mcs_RelatedResourceGroupId.Id);

                        var group = new ConstraintBasedGroup
                        {
                           // Id = McsHelper.getEntRefID("mcs_relatedresourcegroupid"),
                            Id = RelatedConstraintRG.Id,
                            Constraints = builder.ToString(),
                           // Name = McsHelper.getStringValue("mcs_name")
                        };
                        var updateRG = new mcs_resourcegroup
                        {
                            Id = thisId,
                            cvt_resources = resources
                        };

                        OrganizationService.Update(updateRG);
                        OrganizationService.Update(group);
                        whereami = "updated";
                        Logger.WriteDebugMessage("System Resource Updated with " + count + " resources");
                    }

                    if ((RGType == 100000000) || (RGType == 99999999))
                    {

                        var builder = new System.Text.StringBuilder("<Constraints>");
                        builder.Append("<Constraint>");
                        builder.Append("<Expression>");
                        var getResources = from resGroups in srv.mcs_groupresourceSet
                                           join Users in srv.SystemUserSet on resGroups.mcs_RelatedUserId.Id equals Users.Id
                                           where resGroups.mcs_relatedResourceGroupId.Id == thisId && resGroups.statecode == 0
                                           select new
                                           {
                                               //  Users.FirstName,
                                               //  Users.LastName,
                                               //  Users.Id,
                                               resGroups.mcs_RelatedUserId,
                                               resGroups.mcs_name

                                           };

                        string resources = null;
                        var count = 0;
                        foreach (var mcsResourcs in getResources)
                        {
                            //First add the resources to the text field for the view. 
                            resources += mcsResourcs.mcs_RelatedUserId.Name.ToString();
                            resources += " ; ";
                            
                            builder.Append(count > 0 ? " || resource[\"Id\"] == " : "<Body>resource[\"Id\"] == ");
                            builder.Append(mcsResourcs.mcs_RelatedUserId.Id.ToString("B"));
                            count += 1;
                        }
                        if (count == 0)
                        {
                            builder.Append("<Body>false");

                        }

                        builder.Append("</Body>");
                        builder.Append("<Parameters>");
                        builder.Append("<Parameter name=\"resource\" />");
                        builder.Append("</Parameters>");
                        builder.Append("</Expression>");
                        builder.Append("</Constraint>");
                        builder.Append("</Constraints>");

                        var RelatedConstraintRG = srv.ConstraintBasedGroupSet.FirstOrDefault(i => i.Id == RG.mcs_RelatedResourceGroupId.Id);
                        
                        var group = new ConstraintBasedGroup
                        {
                            //Id = McsHelper.getEntRefID("mcs_relatedresourcegroupid"),
                            Id = RelatedConstraintRG.Id,
                            Constraints = builder.ToString(),
                            //Name = McsHelper.getStringValue("mcs_name")
                        };
                        var updateRG = new mcs_resourcegroup
                        {
                            Id = thisId,
                            cvt_resources = resources
                        };

                        OrganizationService.Update(updateRG);
                        OrganizationService.Update(group);
                        whereami = "updated";
                        Logger.WriteDebugMessage("System Resource Updated with " + count + " resources");
                    }
                    if (RGType == 917290000)
                    {
                        var builder = new System.Text.StringBuilder("<Constraints>");
                        builder.Append("<Constraint>");
                        builder.Append("<Expression>");
                        
                        var getResources = from resGroups in srv.mcs_groupresourceSet
                                           join mcsResourcs in srv.mcs_resourceSet on resGroups.mcs_RelatedResourceId.Id equals mcsResourcs.mcs_resourceId.Value
                                           where resGroups.mcs_relatedResourceGroupId.Id == thisId && resGroups.statecode == 0
                                           select new
                                           {
                                               mcsResourcs.mcs_name,
                                               mcsResourcs.mcs_relatedResourceId
                                           };

                        string resources = null;
                        var count = 0;
                        foreach (var mcsResourcs in getResources)
                        {
                            //First add the resources to the text field for the view. 
                            resources += mcsResourcs.mcs_relatedResourceId.Name.ToString();
                            resources += " ; ";
                            
                            builder.Append(count > 0 ? " || resource[\"Id\"] == " : "<Body>resource[\"Id\"] == ");
                            builder.Append(mcsResourcs.mcs_relatedResourceId.Id.ToString("B"));
                            count += 1;
                        }
                        

                        var getUsers = from resGroups in srv.mcs_groupresourceSet
                                           join Users in srv.SystemUserSet on resGroups.mcs_RelatedUserId.Id equals Users.Id
                                           where resGroups.mcs_relatedResourceGroupId.Id == thisId && resGroups.statecode == 0
                                           select new
                                           {
                                               //  Users.FirstName,
                                               //  Users.LastName,
                                               //  Users.Id,
                                               resGroups.mcs_RelatedUserId,
                                               resGroups.mcs_name

                                           };

                        foreach (var User in getUsers)
                        {
                            resources += User.mcs_RelatedUserId.Name.ToString();
                            resources += " ; ";
                            
                            builder.Append(count > 0 ? " || resource[\"Id\"] == " : "<Body>resource[\"Id\"] == ");
                            builder.Append(User.mcs_RelatedUserId.Id.ToString("B"));
                            count += 1;
                        }

                        if (count == 0)
                        {
                            builder.Append("<Body>false");
                        }

                        builder.Append("</Body>");
                        builder.Append("<Parameters>");
                        builder.Append("<Parameter name=\"resource\" />");
                        builder.Append("</Parameters>");
                        builder.Append("</Expression>");
                        builder.Append("</Constraint>");
                        builder.Append("</Constraints>");

                        var RelatedConstraintRG = srv.ConstraintBasedGroupSet.FirstOrDefault(i => i.Id == RG.mcs_RelatedResourceGroupId.Id);

                        var group = new ConstraintBasedGroup
                        {
                            // Id = McsHelper.getEntRefID("mcs_relatedresourcegroupid"),
                            Id = RelatedConstraintRG.Id,
                            Constraints = builder.ToString(),
                            // Name = McsHelper.getStringValue("mcs_name")
                        };

                        var updateRG = new mcs_resourcegroup
                        {
                            Id = thisId,
                            cvt_resources = resources
                        };

                        OrganizationService.Update(updateRG);
                        OrganizationService.Update(group);
                        whereami = "updated";
                        Logger.WriteDebugMessage("System Resource Updated with " + count + " resources");

                    }

                }
                catch (FaultException<OrganizationServiceFault> ex)
                {
                    Logger.WriteToFile(whereami + ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
                catch (Exception ex)
                {
                    Logger.WriteToFile(whereami + ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }
        public override string McsSettingsDebugField
        {
            get { return "mcs_groupresourceplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}
