﻿using System;
using System.ServiceModel;
using Microsoft.Xrm.Sdk;
using mcsScheduling;
using System.Linq;
using MCSShared;

namespace MCSPlugins
{
    public class McsGroupResourceUpdatePostStageRunner : PluginRunner
    {
        #region Constructor
        public McsGroupResourceUpdatePostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }       
        #endregion
        
        #region Internal Methods/Properties
        internal void Execute()
        {
            try
            {
                Logger.setMethod = "Execute";
                Logger.WriteTxnTimingMessage("Starting");

                if (PluginExecutionContext.InputParameters.Contains("Target") && PluginExecutionContext.InputParameters["Target"] is Entity)
                {
                    Logger.WriteDebugMessage(String.Format("At the top of:{0}", GetType()));
                    var entity = (Entity)PluginExecutionContext.InputParameters["Target"];                    
                    var thisGroupResource = entity.ToEntity<mcs_groupresource>();

                    Logger.WriteGranularTimingMessage("Starting UpdateResourceGroup");
                    UpdateResourceGroup(thisGroupResource.Id);
                    Logger.WriteGranularTimingMessage("Ending UpdateResourceGroup");
                }

                Logger.setMethod = "Execute";
                Logger.WriteTxnTimingMessage("Ending");
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }

        internal void UpdateResourceGroup(Guid thisId) //Now Group Resource instead of Resource Group
        {
            Logger.setMethod = "UpdateResourceGroup";
            Logger.WriteDebugMessage("starting UpdateResourceGroup");
            using (var srv = new Xrm(OrganizationService))
            {
                try
                {
                    var thisGroupResource = srv.mcs_groupresourceSet.FirstOrDefault(g => g.Id == thisId);
                    if (thisGroupResource.mcs_relatedResourceGroupId == null)
                        return;

                    var RelatedResourceGroupId = thisGroupResource.mcs_relatedResourceGroupId.Id;
                    var resourceGroup = srv.mcs_resourcegroupSet.FirstOrDefault(i => i.Id == RelatedResourceGroupId);

                    if (resourceGroup == null) 
                        return;

                    Logger.WriteDebugMessage("mcs_resourcegroup was found");

                    var resourceNames = "";
                    var count = 0;
                    var builder = CvtHelper.GetResources(RelatedResourceGroupId, srv, out resourceNames, out count);
                    builder = CvtHelper.BuildConstraintsXML(builder);
                    Logger.WriteDebugMessage("Built XML");
                    
                    //Could check if builder = constraints to see if update even needs to happen
                    if (resourceNames != resourceGroup.cvt_resources)
                    {
                        var resGroupUpdate = new mcs_resourcegroup
                        {
                            Id = resourceGroup.Id,
                            cvt_resources = resourceNames
                        };
                        OrganizationService.Update(resGroupUpdate);
                        Logger.WriteDebugMessage("Updated Resource Group with new string: " + resourceNames);

                        if (resourceGroup.mcs_RelatedResourceGroupId != null)
                        {
                            var group = new ConstraintBasedGroup
                            {
                                Id = resourceGroup.mcs_RelatedResourceGroupId.Id,
                                Constraints = builder.ToString()
                            };
                            OrganizationService.Update(group);
                            Logger.WriteDebugMessage("System Resource Updated with " + count + " resources");
                        }
                    }
                }
                catch (FaultException<OrganizationServiceFault> ex)
                {
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
                catch (Exception ex)
                {
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }
 
    #endregion

        #region Additional Interface Methods
        public override string McsSettingsDebugField
        {
            get { return "mcs_serviceplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}
