﻿using Microsoft.Crm.Sdk.Messages;
using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using mcsScheduling;
using System.Linq;
using Microsoft.Xrm.Sdk.Query;
using MCSShared;

namespace MCSPlugins
{
    public class McsResourceCreateSysRecordsPostStageRunner : PluginRunner
    {
        #region Constructor
        public McsResourceCreateSysRecordsPostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods/Properties
        internal void Execute()
        {
            //This plugin runs after the mcs_resource has been created.
            //
            // It accomplishes the following:
            //1. Create the system service records that will be required for scheduling. 

            try
            {
                Logger.setMethod = "Execute";

                Logger.WriteTxnTimingMessage("Starting");

              //  if (PluginExecutionContext.Depth > 1) { return; }

                if (PluginExecutionContext.InputParameters.Contains("Target") &&
                    PluginExecutionContext.InputParameters["Target"] is Entity)
                {
                    var entity = (Entity)PluginExecutionContext.InputParameters["Target"];
                    var thisResource = entity.ToEntity<mcs_resource>();

                    Logger.WriteDebugMessage(String.Format("At the top of:{0}", GetType()));

                    var resourcespecguid = McsHelper.getStringValue("mcs_resourcespecguid");                    
                    if (resourcespecguid == null)
                    {
                        //Create the system service records
                        Logger.WriteGranularTimingMessage("Starting CreateSysResource");
                        CreateSystemRecords(PluginExecutionContext.PrimaryEntityId);
                        Logger.WriteGranularTimingMessage("Ending CreateSysResource");
                    }
                    var relatedEquip = McsHelper.getEntRefID("mcs_relatedresourceid");
                    var updateCalendar = McsHelper.getBoolValue("cvt_updatecalendar");
                    if ((relatedEquip != null) && (updateCalendar == true))
                    {
                        Logger.WriteGranularTimingMessage("Starting Update System Record Calendar");
                        CvtHelper.ChangeNewlyCreatedCalendar(relatedEquip, OrganizationService, Logger, McsSettings);
                        mcs_resource updateResource = new mcs_resource()
                        {
                            cvt_updatecalendar = false,
                            Id = PluginExecutionContext.PrimaryEntityId
                        };
                        OrganizationService.Update(updateResource);

                        Logger.WriteGranularTimingMessage("Ending Update System Record Caledndar");
                    }
                }

                Logger.setMethod = "Execute";

                Logger.WriteTxnTimingMessage("Ending");

            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));

                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }


        internal void CreateSystemRecords(Guid thisId)
        {
            //Starting the creation of a System Service Records which will be associated with target MCS Resource. 
            Logger.WriteDebugMessage("starting CreateSystemRecords");

            var whereami = "top";

            try
            {
                using (var srv = new Xrm(OrganizationService))
                {
                    Logger.setMethod = "CreateSystemRecords";

                    //build the top half of the constraints xml
                    var builder = new System.Text.StringBuilder("<Constraints>");
                    builder.Append("<Constraint>");
                    builder.Append("<Expression>");
                    builder.Append("<Body>resource[\"Id\"] == ");
                    
                    var resource = srv.mcs_resourceSet.FirstOrDefault(i => i.Id == thisId);
                    if (resource == null) return;
                    Logger.WriteDebugMessage("got resource");
                    builder.Append(resource.mcs_relatedResourceId.Id.ToString("B"));
                        
                    builder.Append("</Body>");
                    builder.Append("<Parameters>");
                    builder.Append("<Parameter name=\"resource\" />");
                    builder.Append("</Parameters>");
                    builder.Append("</Expression>");
                    builder.Append("</Constraint>");
                    builder.Append("</Constraints>");
                    // Define an anonymous type to define the possible constraint based group type code values.
                    var constraintBasedGroupTypeCode = new
                    {
                        Static = 0,
                        Dynamic = 1,
                        Implicit = 2
                    };
                    //we need the user for the business unit
                    Logger.WriteDebugMessage("About to get User");
                    var systemUser = srv.SystemUserSet.FirstOrDefault(i => i.Id == PluginExecutionContext.InitiatingUserId);

                    if (systemUser == null) return;
                    Logger.WriteDebugMessage("got user");

                    var group = new ConstraintBasedGroup
                    {
                        BusinessUnitId = systemUser.BusinessUnitId,
                        Constraints = builder.ToString(),
                        Name = "Selection Rule:" + McsHelper.getStringValue("mcs_name"),
                        GroupTypeCode = new OptionSetValue(constraintBasedGroupTypeCode.Implicit),

                    };

                    var newSysResource = OrganizationService.Create(group);
                    whereami = "created";

                    //now create the resource spec record
                    var spec = new ResourceSpec
                    {
                        BusinessUnitId = systemUser.BusinessUnitId,
                        ObjectiveExpression = @"<Expression><Body>udf ""Random""(factory,resource,appointment,request,leftoffset,rightoffset)</Body><Parameters><Parameter name=""factory"" /><Parameter name=""resource"" /><Parameter name=""appointment"" /><Parameter name=""request"" /><Parameter name=""leftoffset"" /><Parameter name=""rightoffset"" /></Parameters><Properties EvaluationInterval=""P0D"" evaluationcost=""0"" /></Expression>",
                        RequiredCount = 1,
                        Name = "Selection Rule:" + McsHelper.getStringValue("mcs_name"),
                        GroupObjectId = newSysResource,
                        SameSite = true
                    };
                    var _specId = OrganizationService.Create(spec);

                    //update the pat group resource so we can use these values when we create the service.
                    Logger.setMethod = "Update Resource";
                    Logger.WriteDebugMessage("About to update Resource");
                    var updateResource = new Entity("mcs_resource") { Id = thisId };

                    //can't create relationships to these two, so we have to just store the guids in text fields.
                    updateResource["mcs_resourcespecguid"] = _specId.ToString();
                    updateResource["mcs_constraintgroupguid"] = newSysResource.ToString();
                    OrganizationService.Update(updateResource);
                    Logger.WriteDebugMessage("Resource Updated");

                }
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(whereami + ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
            catch (Exception ex)
            {
                Logger.WriteToFile(whereami + ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
        }

        //internal void ChangeNewlyCreatedCalendar(Guid sysResourceId, Guid thisId)
        //{
        //    Logger.WriteDebugMessage("starting ChangeNewlyCreatedCalendar");

        //    var whereami = "top";

        //    using (var srv = new Xrm(OrganizationService))
        //    {
        //        try
        //        {
        //            Logger.setMethod = "ChangeNewlyCreatedCalendar";
        //            int userTimeZone = 0;

        //            var defaultEquipmentfromSettings = McsSettings.GetSingleSetting("mcs_defaultcalendar", "entityreference");


        //            Logger.WriteDebugMessage("Got Default Calendar");
        //            var defaultEquipmentId = new Guid(defaultEquipmentfromSettings);

        //            var defaultEquipment = srv.EquipmentSet.FirstOrDefault(u => u.EquipmentId == defaultEquipmentId);

        //            if (defaultEquipment != null)
        //            {
        //                Logger.WriteDebugMessage("Got Default Equipment");
        //                // Retrieve the calendar of the default resource 
        //                Entity defaultCalendarEntity = OrganizationService.Retrieve("calendar", defaultEquipment.CalendarId.Id, new ColumnSet(true));

        //                // Retrieve the calendar rules defined in the calendar
        //                var newEquipment = srv.EquipmentSet.FirstOrDefault(u => u.EquipmentId == sysResourceId);
        //                if (newEquipment != null)
        //                {
        //                    Logger.WriteDebugMessage("Got new Resource Calendar");

        //                    Logger.WriteDebugMessage(String.Format("Equipment name is {0}", newEquipment.Name));
        //                    // Retrieve the calendar of the default resource 
        //                    Entity newCalendarEntity = OrganizationService.Retrieve("calendar", newEquipment.CalendarId.Id, new ColumnSet(true));
        //                    // Retrieve the calendar rules defined in the calendar
        //                    EntityCollection newcalendarRules = (EntityCollection)newCalendarEntity.Attributes["calendarrules"];

        //                    var newTimeCode = 0;
        //                    EntityCollection defaultCalendarRules = (EntityCollection)defaultCalendarEntity.Attributes["calendarrules"];
        //                    var i = 0;
        //                    foreach (CalendarRule defaultCalRule in defaultCalendarRules.Entities)
        //                    {

        //                        if (newcalendarRules.Entities.Count >= (i + 1))
        //                        {
        //                            Logger.WriteDebugMessage("Creating new calendar:" + i);
        //                            CalendarRule newRule = (newcalendarRules.Entities[i]).ToEntity<CalendarRule>();
        //                            newRule.Duration = defaultCalRule.Duration;
        //                            newRule.EffectiveIntervalEnd = defaultCalRule.EffectiveIntervalEnd;
        //                            newRule.EffectiveIntervalStart = defaultCalRule.EffectiveIntervalStart;
        //                            newRule.StartTime = defaultCalRule.StartTime;
        //                            newRule.Pattern = defaultCalRule.Pattern;
        //                            newTimeCode = newRule.TimeZoneCode.Value;

        //                         //   if (i == 0)
        //                         //   {
        //                                if (defaultCalRule.InnerCalendarId != null)
        //                                {
        //                                    Entity defaultInnerCalendarEntity = OrganizationService.Retrieve("calendar", defaultCalRule.InnerCalendarId.Id, new ColumnSet(true));
        //                                    Entity newInnerCalendarEntity = OrganizationService.Retrieve("calendar", newRule.InnerCalendarId.Id, new ColumnSet(true));

        //                                    EntityCollection newInnerCalendarRules = (EntityCollection)newInnerCalendarEntity.Attributes["calendarrules"];
        //                                    EntityCollection defaultInnerCalendarRules = (EntityCollection)defaultInnerCalendarEntity.Attributes["calendarrules"];

        //                                    //there is always 1 and only 1 inner calendar, but there may be many more than 1 outer calendar.
        //                                    CalendarRule newInnerCalRule = (newInnerCalendarRules.Entities[i]).ToEntity<CalendarRule>();
        //                                    CalendarRule defaultInnerCalRule = (defaultInnerCalendarRules.Entities[i]).ToEntity<CalendarRule>();

        //                                    newInnerCalRule.Effort = defaultInnerCalRule.Effort;
        //                                    newInnerCalRule.Offset = defaultInnerCalRule.Offset;
        //                                    newInnerCalRule.Duration = defaultInnerCalRule.Duration;

        //                                    OrganizationService.Update(newInnerCalendarEntity);
        //                                    Logger.WriteDebugMessage("Updating Calendar");
        //                                }
        //                          //  }
        //                        }
        //                        else
        //                        {
        //                            //this means that the default calendar has more than 1 calendar entry.
        //                            if (defaultCalRule.Name == "Holiday Closure Link")
        //                            {
        //                                //this means the resource is following business closures.  Copy EVERYTHING from this one.
        //                                CalendarRule newRule = new CalendarRule();
        //                                newRule.Rank = defaultCalRule.Rank;
        //                                newRule.StartTime = defaultCalRule.StartTime;
        //                                newRule.Name = defaultCalRule.Name;
        //                                newRule.TimeZoneCode = newTimeCode;
        //                                newRule.ExtentCode = defaultCalRule.ExtentCode;
        //                                newRule.EffectiveIntervalEnd = defaultCalRule.EffectiveIntervalEnd;
        //                                newRule.Pattern = defaultCalRule.Pattern;
        //                                newRule.InnerCalendarId = defaultCalRule.InnerCalendarId;
        //                                newRule.GroupDesignator = defaultCalRule.GroupDesignator;
        //                                newRule.Duration = defaultCalRule.Duration;
        //                                newRule.Description = "Default";

        //                                newcalendarRules.Entities.Add(newRule);
        //                                Logger.WriteDebugMessage("Updating Calendar to include holidays");
        //                            }
        //                            else
        //                            {
        //                                // Create a new  calendar
        //                                Entity newCalendar = new Entity("calendar");
        //                                newCalendar.Attributes["businessunitid"] = new EntityReference("businessunit", ((Microsoft.Xrm.Sdk.EntityReference)(newCalendarEntity["businessunitid"])).Id);
        //                                newCalendar.Attributes["description"] = defaultCalendarEntity["description"];
        //                                Guid innerCalendarId = OrganizationService.Create(newCalendar);

        //                                // Create a new calendar rule and assign the inner calendar id to it
        //                                CalendarRule calendarRule = new CalendarRule();
        //                                calendarRule.ExtentCode = defaultCalRule.ExtentCode;
        //                                calendarRule.StartTime = defaultCalRule.StartTime;
        //                                calendarRule.TimeZoneCode = newTimeCode;
        //                                calendarRule.Pattern = defaultCalRule.Pattern;
        //                                calendarRule.Duration = defaultCalRule.Duration;
        //                                calendarRule.Rank = defaultCalRule.Rank;
        //                                calendarRule.InnerCalendarId = new EntityReference("calendar", innerCalendarId);

        //                                newcalendarRules.Entities.Add(calendarRule);
        //                                Logger.WriteDebugMessage("Adding new innerCalendar");

        //                                if (defaultCalRule.InnerCalendarId != null)
        //                                {
        //                                    Entity defaultInnerCalendarEntity = OrganizationService.Retrieve("calendar", defaultCalRule.InnerCalendarId.Id, new ColumnSet(true));

        //                                    EntityCollection defaultInnerCalendarRules = (EntityCollection)defaultInnerCalendarEntity.Attributes["calendarrules"];

        //                                    //there is always 1 and only 1 inner calendar, but there may be many more than 1 outer calendar.
        //                                    CalendarRule defaultInnerCalRule = (defaultInnerCalendarRules.Entities[0]).ToEntity<CalendarRule>();

        //                                    CalendarRule newRule = new CalendarRule();
        //                                    newRule.Description = "Default";
        //                                    newRule.Rank = defaultInnerCalRule.Rank;
        //                                    newRule.StartTime = defaultInnerCalRule.StartTime;
        //                                    newRule.Name = defaultInnerCalRule.Name;
        //                                    newRule.TimeZoneCode = newTimeCode;
        //                                    newRule.Offset = defaultInnerCalRule.Offset;
        //                                    newRule.ExtentCode = defaultInnerCalRule.ExtentCode;
        //                                    newRule.EffectiveIntervalEnd = defaultInnerCalRule.EffectiveIntervalEnd;
        //                                    newRule.IsSimple = defaultInnerCalRule.IsSimple;
        //                                    newRule.GroupDesignator = defaultInnerCalRule.GroupDesignator;
        //                                    newRule.Duration = defaultInnerCalRule.Duration;
        //                                    newRule.TimeCode = defaultInnerCalRule.TimeCode;
        //                                    newRule.SubCode = defaultInnerCalRule.SubCode;


        //                                    EntityCollection innerCalendarRules = new EntityCollection();
        //                                    innerCalendarRules.EntityName = "calendarrule";
        //                                    innerCalendarRules.Entities.Add(newRule);

        //                                    newCalendar.Attributes["calendarrules"] = innerCalendarRules;
        //                                    newCalendar.Attributes["calendarid"] = innerCalendarId;

        //                                    OrganizationService.Update(newCalendar);
        //                                    Logger.WriteDebugMessage("Service Update Inner");
        //                                }
        //                            }

        //                        }
        //                        i += 1;
        //                    }
        //                    OrganizationService.Update(newCalendarEntity);
        //                    Logger.WriteDebugMessage("Service Update Outer");
        //                }

        //                //If the Calendar Update method was run, we will set the Update Calendar boolean field back to No for next time. 
        //                var updateResource = new Entity("mcs_resource") { Id = thisId };                    
        //                updateResource["cvt_updatecalendar"] = false;                        
        //                OrganizationService.Update(updateResource);
        //                Logger.WriteDebugMessage("Resource Updated");
        //            }
        //        }
        //        catch (FaultException<OrganizationServiceFault> ex)
        //        {
        //            Logger.WriteToFile(whereami + ex.Message);
        //            throw new InvalidPluginExecutionException(ex.Message);
        //        }
        //        catch (Exception ex)
        //        {
        //            Logger.WriteToFile(whereami + ex.Message);
        //            throw new InvalidPluginExecutionException(ex.Message);
        //        }
        //    }
        //}
                   
        public override string McsSettingsDebugField
        {
            get { return "mcs_resourceplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}
