﻿using Microsoft.Crm.Sdk.Messages;
using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using mcsScheduling;
using System.Linq;
using Microsoft.Xrm.Sdk.Query;

namespace MCSPlugins
{
    public class McsResourceGroupCreatePostStageRunner : PluginRunner
    {
        #region Constructor
        public McsResourceGroupCreatePostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods/Properties
        internal void Execute()
        {
            try
            {
                Logger.setMethod = "Execute";

                Logger.WriteTxnTimingMessage("Starting");

                if (PluginExecutionContext.InputParameters.Contains("Target") &&
                    PluginExecutionContext.InputParameters["Target"] is Entity)
                {

                    Logger.WriteDebugMessage(String.Format("At the top of:{0}", GetType()));

                    Logger.WriteGranularTimingMessage("Starting CreateSysGroupResource");
                    //regardless of quick create - let's do the creation of the system resource group.
                    CreateSysGroupResource(PluginExecutionContext.PrimaryEntityId, PluginExecutionContext.InitiatingUserId);
                    Logger.WriteGranularTimingMessage("Ending CreateSysGroupResource");
                    
                    //if the mcs_tsaguid is present then it is because they are doing a quick create
                    var quickCreateOption = McsHelper.getStringValue("mcs_tsaguid");
                    //but we have to know which one, pat or prov.
                    //These 3 fields are populated from jscript on a ribbon button.
                    var quickCreatePat = McsHelper.getBoolValue("mcs_createpatientrg");
                    var quickCreateProv = McsHelper.getBoolValue("mcs_createproviderrg");

                    var doQuickCreate = quickCreateOption != null;
                    Logger.WriteDebugMessage(String.Format("doQuickCreate:{0}{1}", doQuickCreate, quickCreateOption));
                    if (!doQuickCreate)
                    {
                        Logger.WriteDebugMessage("Not doing Quick Create for TSA, maybe MTSA");
                        //if the mcs_mastertsaguid is present then it is because they are doing a quick create
                        quickCreateOption = McsHelper.getStringValue("cvt_mastertsaguid");

                        doQuickCreate = quickCreateOption != null;
                    }
                    Logger.WriteDebugMessage(String.Format("doQuickCreate:{0}{1}", doQuickCreate, quickCreateOption));

                    if (doQuickCreate)
                    {
                        Logger.WriteDebugMessage("about to do quick create");

                        if (quickCreatePat)
                        {
                            Logger.WriteGranularTimingMessage("Starting QuickCreatePatientGroupResource");

                            //call the method to do a quick create for pat group
                            QuickCreatePatientGroupResource(PluginExecutionContext.PrimaryEntityId);

                            Logger.WriteGranularTimingMessage("Ending QuickCreatePatientGroupResource");
                            
                        }
                        else
                        {
                            if (quickCreateProv)
                            {
                                Logger.WriteGranularTimingMessage("Starting QuickCreateProviderGroupResource");

                                //call the method to do a quick create for provider group
                                QuickCreateProviderGroupResource(PluginExecutionContext.PrimaryEntityId);

                                Logger.WriteGranularTimingMessage("Ending QuickCreateProviderGroupResource");
                            }
                        }
                    }
                    else
                    {
                        Logger.WriteDebugMessage("not doing quick create");
                    }
                  
                }
                Logger.setMethod = "Execute";
                Logger.WriteTxnTimingMessage("Ending");

            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));

                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }

        }

        internal void QuickCreatePatientGroupResource(Guid primaryEntityId)
        {
            try
            {
                Logger.setMethod = "QuickCreatePatientGroupResource";
         
                var newGroupResource = new Entity("cvt_patientresourcegroup");
                newGroupResource["cvt_tsaresourcetype"] = new OptionSetValue(0);

                newGroupResource["cvt_type"] = new OptionSetValue(McsHelper.getOptionSetValue("mcs_type"));

                var tsapresent = McsHelper.getStringValue("mcs_tsaguid");
                if (tsapresent != null)
                {
                    newGroupResource["cvt_relatedtsaid"] = new EntityReference("mcs_services", new Guid(tsapresent));
                }

                newGroupResource["cvt_relatedsiteid"] = new EntityReference("mcs_site", 
                    McsHelper.getEntRefID("mcs_relatedsiteid"));

                newGroupResource["cvt_relatedresourcegroupid"] = new EntityReference("mcs_resourcegroup", primaryEntityId);

                newGroupResource["cvt_name"] = McsHelper.getStringValue("mcs_name");

                OrganizationService.Create(newGroupResource);

                Logger.WriteDebugMessage("created patient resource group record");
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);

            }
            catch (Exception ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);

            }
        }
        internal void QuickCreateProviderGroupResource(Guid primaryEntityId)
        {
            try
            {
                Logger.setMethod = "QuickCreateProviderGroupResource";
         
                var newGroupResource = new Entity("cvt_providerresourcegroup");
                newGroupResource["cvt_tsaresourcetype"] = new OptionSetValue(0);

                newGroupResource["cvt_type"] = new OptionSetValue(McsHelper.getOptionSetValue("mcs_type"));


                var mastertsapresent = McsHelper.getStringValue("cvt_mastertsaguid");
                if (mastertsapresent != null)
                {
                    newGroupResource["cvt_relatedmastertsaid"] = new EntityReference("cvt_mastertsa", new Guid(mastertsapresent));
                }

                var tsapresent = McsHelper.getStringValue("mcs_tsaguid");
                if (tsapresent != null)
                {
                    newGroupResource["cvt_relatedtsaid"] = new EntityReference("mcs_services", new Guid(tsapresent));
                }

                newGroupResource["cvt_relatedsiteid"] = new EntityReference("mcs_site",
                    McsHelper.getEntRefID("mcs_relatedsiteid"));

                newGroupResource["cvt_relatedresourcegroupid"] = new EntityReference("mcs_resourcegroup", primaryEntityId);

                newGroupResource["cvt_name"] = McsHelper.getStringValue("mcs_name");
                Logger.WriteDebugMessage("About to Create Prov Group Resource group");
                OrganizationService.Create(newGroupResource);

                Logger.WriteDebugMessage("created provider resource group record");
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);

            }
            catch (Exception ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);

            }
        }
        internal void CreateSysGroupResource(Guid thisId, Guid userId)
        {
            Logger.WriteDebugMessage("starting CreateSysGroupResource");

            var whereami = "top";

            using (var srv = new Xrm(OrganizationService))
            {
                try
                {
                    Logger.setMethod = "Create System Group Resource";

                    var thisRG = srv.mcs_resourcegroupSet.FirstOrDefault(i => i.Id == thisId);

                    System.Text.StringBuilder builder = new System.Text.StringBuilder("<Constraints>");
                    builder.Append("<Constraint>");
                    builder.Append("<Expression>");
                    //Added
                    builder.Append("<Body>resource[\"Id\"] == ");
                    builder.Append(thisId.ToString("B"));

                    //removed
                    //builder.Append("<Body>false");
                    builder.Append("</Body>");
                    builder.Append("<Parameters>");
                    builder.Append("<Parameter name=\"resource\" />");
                    builder.Append("</Parameters>");
                    builder.Append("</Expression>");
                    builder.Append("</Constraint>");
                    builder.Append("</Constraints>");

                    // Define an anonymous type to define the possible constraint based group type code values.
                    var constraintBasedGroupTypeCode = new
                    {
                        Static = 0,
                        Dynamic = 1,
                        Implicit = 2
                    };

                    Logger.WriteDebugMessage("About to get User");
                    var systemUser = srv.SystemUserSet.FirstOrDefault(i => i.Id == userId);
                    if (systemUser == null) return;
                    Logger.WriteDebugMessage("got user");

                    int GroupTypeCode;
                    var RGType = thisRG.mcs_Type.Value;
                    if (RGType == 917290000)
                    {
                        GroupTypeCode = constraintBasedGroupTypeCode.Static;
                    }
                    else
                    {
                        GroupTypeCode = constraintBasedGroupTypeCode.Dynamic;
                    }

                    var group = new ConstraintBasedGroup
                    {
                        
                        BusinessUnitId = systemUser.BusinessUnitId,
                        Name = McsHelper.getStringValue("mcs_name"),
                        Constraints = builder.ToString(),                    
                        GroupTypeCode = new OptionSetValue(GroupTypeCode),
                                          
                    };
                    var newSysResource = OrganizationService.Create(group);
                    whereami = "created";
                    Logger.WriteDebugMessage("System Resource Added");

                    var selectiongroup = new ConstraintBasedGroup
                    {
                        BusinessUnitId = systemUser.BusinessUnitId,
                        Constraints = builder.ToString(),
                        Name = "Selection Rule:" + McsHelper.getStringValue("cvt_name"),
                        GroupTypeCode = new OptionSetValue(constraintBasedGroupTypeCode.Implicit),

                    };
                    var newSelectionGroup = OrganizationService.Create(selectiongroup);
                    whereami = "created";
                    Logger.WriteDebugMessage("Selection Group Added");

                    int reqCount;
                    if (RGType == 917290000)
                    {
                        reqCount = -1;
                    }
                    else
                    {
                        reqCount = 1;
                    }

                    var spec = new ResourceSpec
                    {
                        BusinessUnitId = systemUser.BusinessUnitId,
                        ObjectiveExpression = @"<Expression><Body>udf ""Random""(factory,resource,appointment,request,leftoffset,rightoffset)</Body><Parameters><Parameter name=""factory"" /><Parameter name=""resource"" /><Parameter name=""appointment"" /><Parameter name=""request"" /><Parameter name=""leftoffset"" /><Parameter name=""rightoffset"" /></Parameters><Properties EvaluationInterval=""P0D"" evaluationcost=""0"" /></Expression>",
                        RequiredCount = reqCount,
                        Name = "Selection Rule:" + McsHelper.getStringValue("mcs_name"),
                        GroupObjectId = newSysResource,
                        SameSite = true
                    };
                    var _specId = OrganizationService.Create(spec);
                    
                    Logger.setMethod = "Update MCS Resource Group";
                    Logger.WriteDebugMessage("About to update MCS Resource Group");
                    var updateResourceGroup = new Entity("mcs_resourcegroup") { Id = thisId };
                    updateResourceGroup["mcs_relatedresourcegroupid"] = new EntityReference(ConstraintBasedGroup.EntityLogicalName, newSysResource);
                    updateResourceGroup["mcs_constraintgroupguid"] = newSelectionGroup.ToString();
                    updateResourceGroup["mcs_resourcespecguid"] = _specId.ToString();
                    OrganizationService.Update(updateResourceGroup);
                    Logger.WriteDebugMessage("MCS Resource Group Updated");

                    ColumnSet attributes = new ColumnSet(new string[] { "cvt_primaryteam" });

                    // Retrieve the system user's primary team
                    Entity initiatingUserEntity = OrganizationService.Retrieve("systemuser", PluginExecutionContext.InitiatingUserId, attributes);
                    if (initiatingUserEntity.Attributes.Contains("cvt_primaryteam"))
                    {
                        var myTeam = (EntityReference)initiatingUserEntity["cvt_primaryteam"];
                        AssignRequest assignRequest = new AssignRequest()
                        {
                            Assignee = new EntityReference
                            {
                                LogicalName = Team.EntityLogicalName,
                                Id = myTeam.Id
                            },

                            Target = new EntityReference("mcs_resourcegroup", thisId)
                        };

                        OrganizationService.Execute(assignRequest);


                    }

                }
                catch (FaultException<OrganizationServiceFault> ex)
                {
                    Logger.WriteToFile(whereami + ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
                catch (Exception ex)
                {
                    Logger.WriteToFile(whereami + ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }

        public override string McsSettingsDebugField
        {
            get { return "mcs_resourcegroupplugin"; }
        }

        //For Create plugins there is no PRE image.
        //for anything else PRE images can exist on a pre-stage plugin but POST image does not.
        // For any other POST stage plugins PRE and POST exist.

        //since target only contains what was changed - the purpose of the helper is to be able to get a value of a field that may or may not have been updated.
        //For pre-stage plugins this can be difficult and require lots of coding as you want to use the target if the user updated the field and the property exists in the target - but if it
        //doesn't exist then the pre-image would hold the value.  That can be 4 lines of code, or more, for each attribute.

        //however for a post stage plugin if you want the post value then make the thisEntity and preEntity both contain the post image.
        //though not common it is possible for a post image to be different than the target.  
        //One scenario is phone number where maybe the phone number in target is 123-4567890 and the post it is 
        //(123) 456-7890.  This happens if a plugin changes the value prior to it getting to the db.

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}
