﻿using System.Linq;
using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using mcsScheduling;

namespace MCSPlugins
{
    public class McsResourceGroupUpdatePostStageRunner : PluginRunner
    {
        #region Constructor
        public McsResourceGroupUpdatePostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods/Properties
        internal void Execute()
        {
            try
            {
                if (PluginExecutionContext.Depth > 1) { return; }
                
                Logger.setMethod = "Execute";
                Logger.WriteTxnTimingMessage("Starting");

                if (PluginExecutionContext.InputParameters.Contains("Target") &&
                    PluginExecutionContext.InputParameters["Target"] is Entity)
                {

                    var entity = (Entity)PluginExecutionContext.InputParameters["Target"];
                    var thisResourceGroup = entity.ToEntity<mcs_resourcegroup>();

                    //We will check these Boolean fields first to see if they have been flagged for Update. 
                    bool updateResourceGroup = false;

                    if (thisResourceGroup.cvt_updateresourcegroup != null) { updateResourceGroup = thisResourceGroup.cvt_updateresourcegroup.Value; }
                    if (updateResourceGroup == true)
                    {
                        Logger.WriteDebugMessage(String.Format("At the top of:{0}", GetType()));
                        Logger.WriteGranularTimingMessage("Starting UpdateSystemResource");
                        UpdateResourceGroup(PluginExecutionContext.PrimaryEntityId);
                        Logger.WriteGranularTimingMessage("Ending UpdateSytemResource");
                    }
                }
                else
                {
                    Logger.WriteDebugMessage("Not Doing Update of Resource Group");
                }

                Logger.setMethod = "Execute";
                Logger.WriteTxnTimingMessage("Ending");            
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
                throw new InvalidPluginExecutionException(ex.Message);
            }
            catch (Exception ex)
            {
                if (ex.Message.StartsWith("custom"))
                {
                    Logger.WriteDebugMessage(ex.Message.Substring(6));
                    throw new InvalidPluginExecutionException(ex.Message.Substring(6));
                }
                else
                {
                    Logger.setMethod = "Execute";
                    Logger.WriteToFile(ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }

        internal void UpdateResourceGroup(Guid thisId)
        {
            Logger.WriteDebugMessage("starting UpdateResourceGroup");
            var whereami = "top";
            using (var srv = new Xrm(OrganizationService))
            {
                try
                {
                    Logger.setMethod = "Update System Group Resource";
                    var builder = new System.Text.StringBuilder("<Constraints>");
                    builder.Append("<Constraint>");
                    builder.Append("<Expression>");
                    var getResources = from resGroups in srv.mcs_groupresourceSet
                                       join mcsResourcs in srv.mcs_resourceSet on resGroups.mcs_RelatedResourceId.Id equals mcsResourcs.mcs_resourceId.Value
                                       where resGroups.mcs_relatedResourceGroupId.Id == thisId && resGroups.statecode == 0
                                       select new
                                       {
                                           mcsResourcs.mcs_name,
                                           mcsResourcs.mcs_relatedResourceId,
                                           resGroups.mcs_groupresourceId
                                       };

                    string resources = null;
                    var count = 0;
                    foreach (var mcsResourcs in getResources)
                    {
                        if (mcsResourcs.mcs_groupresourceId.Value != thisId)
                        {
                            //First add the resources to the text field for the view. 
                            resources += mcsResourcs.mcs_relatedResourceId.Name.ToString();
                            resources += " ; ";

                            builder.Append(count > 0 ? " || resource[\"Id\"] == " : "<Body>resource[\"Id\"] == ");
                            builder.Append(mcsResourcs.mcs_relatedResourceId.Id.ToString("B"));
                            count += 1;
                        }
                    }

                    var getUsers = from resGroups in srv.mcs_groupresourceSet
                                   join Users in srv.SystemUserSet on resGroups.mcs_RelatedUserId.Id equals Users.Id
                                   where resGroups.mcs_relatedResourceGroupId.Id == thisId && resGroups.statecode == 0
                                   select new
                                   {
                                       //  Users.FirstName,
                                       //  Users.LastName,
                                       //  Users.Id,
                                       resGroups.mcs_RelatedUserId,
                                       resGroups.mcs_name,
                                       resGroups.mcs_groupresourceId
                                   };

                    foreach (var User in getUsers)
                    {
                        if (User.mcs_groupresourceId.Value != thisId)
                        {
                            resources += User.mcs_RelatedUserId.Name.ToString();
                            resources += " ; ";

                            builder.Append(count > 0 ? " || resource[\"Id\"] == " : "<Body>resource[\"Id\"] == ");
                            builder.Append(User.mcs_RelatedUserId.Id.ToString("B"));
                            count += 1;
                        }
                    }

                    if (count == 0)
                    {
                        builder.Append("<Body>false");
                    }

                    builder.Append("</Body>");
                    builder.Append("<Parameters>");
                    builder.Append("<Parameter name=\"resource\" />");
                    builder.Append("</Parameters>");
                    builder.Append("</Expression>");
                    builder.Append("</Constraint>");
                    builder.Append("</Constraints>");

                    var thisResourceGroup = srv.mcs_resourcegroupSet.FirstOrDefault(i => i.Id == thisId);
                    if (thisResourceGroup.mcs_RelatedResourceGroupId != null)
                    {
                        var myRealResourceGroup = (EntityReference)thisResourceGroup["mcs_relatedresourcegroupid"];
                        var group = new ConstraintBasedGroup
                        {
                            Id = myRealResourceGroup.Id,
                            Constraints = builder.ToString()
                        };

                        var entity = (Entity)PluginExecutionContext.InputParameters["Target"];
                        if (entity.Attributes.Contains("mcs_name"))
                        {
                            var name = (String)entity["mcs_name"];
                            var updateRG = new mcs_resourcegroup
                            {

                                Id = thisId,
                                mcs_name = name, //((Entity)PluginExecutionContext.InputParameters["Target"]).Attributes["mcs_name"].ToString(),
                                cvt_resources = resources
                            };

                        OrganizationService.Update(updateRG);
                        OrganizationService.Update(group);
                        Logger.WriteDebugMessage("System Resource Updated with " + count + " resources");
                        }                     
                    }
                 }
                
                catch (FaultException<OrganizationServiceFault> ex)
                {
                    Logger.WriteToFile(whereami + ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
                catch (Exception ex)
                {
                    Logger.WriteToFile(whereami + ex.Message);
                    throw new InvalidPluginExecutionException(ex.Message);
                }
            }
        }
        #endregion

        #region Additional Interface Methods/Properties
        public override string McsSettingsDebugField
        {
            get { return "mcs_resourcegroupplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}