﻿using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;
using mcsScheduling;
using System.Linq;
using MCSShared;

namespace MCSPlugins
{
    public class McsSiteUpdatePostStageRunner : PluginRunner
    {
        #region Constructor
        public McsSiteUpdatePostStageRunner(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
        }
        #endregion

        #region Internal Methods/Properties
        internal void Execute()
        {         
            try
            {
                if (PluginExecutionContext.Depth > 1) { return; }
                Logger.setMethod = "Execute";
                Logger.WriteDebugMessage("Starting");
                if (PluginExecutionContext.InputParameters.Contains("Target") && PluginExecutionContext.InputParameters["Target"] is Entity)
                {
                    Entity siteContext = ((Entity)(PluginExecutionContext.InputParameters["Target"])).ToEntity<mcs_site>();
                    using (var srv = new Xrm(OrganizationService))
                    {
                        var thisSite = srv.mcs_siteSet.FirstOrDefault(i => i.Id == PluginExecutionContext.PrimaryEntityId);
                        if (thisSite == null)                         
                            return;

                        //Limit calling Update System Site function to change in name, timezone 
                        if ((siteContext.Contains("mcs_name")) || (siteContext.Contains("mcs_timezone")))
                            UpdateSystemSite(McsHelper.getEntRefID("mcs_relatedactualsiteid"));
                        else
                            Logger.WriteDebugMessage("No need to update System Site record");
                        
                        //Limit calling Align Locations to change in Facility
                        if (siteContext.Contains("mcs_facilityid"))
                            CvtHelper.AlignLocations(thisSite, OrganizationService, Logger);
                        else
                            Logger.WriteDebugMessage("No need to Align Locations");
                    }                     
                }
                Logger.setMethod = "Execute";
                Logger.WriteDebugMessage("Ending");
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                Logger.WriteToFile(ex.Message);
            }
            catch (Exception ex)
            {
                Logger.WriteToFile(ex.Message);
            }        
        }

        internal void UpdateSystemSite(Guid id)
        {
            using (var srv = new Xrm(OrganizationService))
            {
                Logger.setMethod = "UpdateSystemSite";
                Logger.WriteTxnTimingMessage("starting UpdateSystemSite");
                var systemSite = srv.SiteSet.FirstOrDefault(i => i.Id == id);
                if (systemSite == null) return;
                Logger.WriteTxnTimingMessage("Retrieved system site");

                var updateSystemSite = new Site
                {
                    Id = systemSite.Id,
                    Name = McsHelper.getStringValue("mcs_name"),
                    TimeZoneCode = McsHelper.getIntValue("mcs_timezone"),
                };

                OrganizationService.Update(updateSystemSite);
                Logger.WriteDebugMessage("System Site record updated");
            }
        }
        #endregion

        #region Additional Interface Methods/Properties
        public override string McsSettingsDebugField
        {
            get { return "mcs_siteplugin"; }
        }

        public override Entity GetPrimaryEntity()
        {
            return (Entity)PluginExecutionContext.InputParameters["Target"];
        }

        public override Entity GetSecondaryEntity()
        {
            return (Entity)PluginExecutionContext.PostEntityImages["post"];
        }
        #endregion
    }
}