﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Activities;
using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Workflow;
using mcsScheduling;

namespace WorkflowActivities
{
    public sealed class GetFacility : CodeActivity
    {
        [Input("SiteReference")]
        [ReferenceTarget("mcs_site")]
        public InArgument<EntityReference> SiteName { get; set; }

        [Output("Facility")]
        [ReferenceTarget("mcs_facility")]
        public OutArgument<EntityReference> FacilityName { get; set; }


        protected override void Execute(CodeActivityContext executionContext)
        {
            try
            {
                mcs_facility facility = null;
                Guid siteId = new Guid();
                //MCSLogger _Logger;

                IWorkflowContext context = executionContext.GetExtension<IWorkflowContext>();

                //Create an Organization Service
                IOrganizationServiceFactory serviceFactory = executionContext.GetExtension<IOrganizationServiceFactory>();
                IOrganizationService service = serviceFactory.CreateOrganizationService(context.InitiatingUserId);

                EntityReference siteRef = SiteName.Get<EntityReference>(executionContext);
                if (siteRef != null)
                    siteId = siteRef.Id;
                using (var srv = new Xrm(service))
                {
                    mcs_site site = srv.mcs_siteSet.FirstOrDefault(s => s.Id == siteId);
                    facility = srv.mcs_facilitySet.FirstOrDefault(f => f.Id == site.mcs_FacilityId.Id);
                }
                if (facility != null)
                {
                    FacilityName.Set(executionContext, new EntityReference()
                    {
                        LogicalName = mcs_facility.EntityLogicalName,
                        Id = facility.Id,
                        Name = facility.mcs_name
                    });
                }
                else
                {
                    FacilityName.Set(executionContext, null);
                }
            }
            catch (Exception ex)
            {
                ITracingService tracingService = executionContext.GetExtension<ITracingService>();
                tracingService.Trace("unable to find facility based on site: " + ex.Message);
                FacilityName.Set(executionContext, null);
            }
        }
    }
}
